///////////////////////////////////////////////////////////////////////////////
//                                       MD5lib 
//                                     (ver. 0.3)
//
//  test:
//  gcc -o md5 -O3 -lm md5lib.c
//
///////////////////////////////////////////////////////////////////////////////




#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <stdint.h>
#include <math.h>




#define TEST 0
#define LEFTROTATE(x, c) (((x) << (c)) | ((x) >> (32 - (c))))


unsigned int h0, h1, h2, h3;
 


void md5(unsigned char *initial_msg, size_t initial_len) {
 
   unsigned char *msg = NULL;
 
   unsigned int r[] = {7, 12, 17, 22, 7, 12, 17, 22, 7, 12, 17, 22, 7, 12, 17, 22,
                       5,  9, 14, 20, 5,  9, 14, 20, 5,  9, 14, 20, 5,  9, 14, 20,
                       4, 11, 16, 23, 4, 11, 16, 23, 4, 11, 16, 23, 4, 11, 16, 23,
                       6, 10, 15, 21, 6, 10, 15, 21, 6, 10, 15, 21, 6, 10, 15, 21};

   unsigned int k[] = {
        0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee,
        0xf57c0faf, 0x4787c62a, 0xa8304613, 0xfd469501,
        0x698098d8, 0x8b44f7af, 0xffff5bb1, 0x895cd7be,
        0x6b901122, 0xfd987193, 0xa679438e, 0x49b40821,
        0xf61e2562, 0xc040b340, 0x265e5a51, 0xe9b6c7aa,
        0xd62f105d, 0x02441453, 0xd8a1e681, 0xe7d3fbc8,
        0x21e1cde6, 0xc33707d6, 0xf4d50d87, 0x455a14ed,
        0xa9e3e905, 0xfcefa3f8, 0x676f02d9, 0x8d2a4c8a,
        0xfffa3942, 0x8771f681, 0x6d9d6122, 0xfde5380c,
        0xa4beea44, 0x4bdecfa9, 0xf6bb4b60, 0xbebfbc70,
        0x289b7ec6, 0xeaa127fa, 0xd4ef3085, 0x04881d05,
        0xd9d4d039, 0xe6db99e5, 0x1fa27cf8, 0xc4ac5665,
        0xf4292244, 0x432aff97, 0xab9423a7, 0xfc93a039,
        0x655b59c3, 0x8f0ccc92, 0xffeff47d, 0x85845dd1,
        0x6fa87e4f, 0xfe2ce6e0, 0xa3014314, 0x4e0811a1,
        0xf7537e82, 0xbd3af235, 0x2ad7d2bb, 0xeb86d391};
 
   h0 = 0x67452301;
   h1 = 0xefcdab89;
   h2 = 0x98badcfe;
   h3 = 0x10325476;
 
   int new_len;
   for(new_len = initial_len*8 + 1; new_len%512!=448; new_len++);
   new_len /= 8;
 
   msg = calloc(new_len + 64, 1); 
                                  
   memcpy(msg, initial_msg, initial_len);
   msg[initial_len] = 128;
 
   unsigned int bits_len = 8*initial_len;
   memcpy(msg + new_len, &bits_len, 4);
 
   int offset;
   for(offset=0; offset<new_len; offset += (512/8)) {
 
   unsigned int *w = (unsigned int *) (msg + offset);
 
#ifdef DEBUG
        if(TEST) printf("offset: %d %x\n", offset, offset);
        int j;
        for(j =0; j < 64; j++) if(TEST) printf("%x ", ((unsigned char *) w)[j]);
        puts("");
#endif
 
        unsigned int a = h0;
        unsigned int b = h1;
        unsigned int c = h2;
        unsigned int d = h3;
 
        unsigned int i;
        for(i = 0; i<64; i++) {

#ifdef ROUNDS
            unsigned char *p;
            if(TEST) printf("%i: ", i);
            p=(unsigned char *)&a;
            if(TEST) printf("%2.2x%2.2x%2.2x%2.2x ", p[0], p[1], p[2], p[3], a);
         
            p=(unsigned char *)&b;
            if(TEST) printf("%2.2x%2.2x%2.2x%2.2x ", p[0], p[1], p[2], p[3], b);
         
            p=(unsigned char *)&c;
            if(TEST) printf("%2.2x%2.2x%2.2x%2.2x ", p[0], p[1], p[2], p[3], c);
         
            p=(unsigned char *)&d;
            if(TEST) printf("%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3], d);
            puts("");
#endif        

            unsigned int f, g;
 
             if (i < 16) {
                f = (b & c) | ((~b) & d);
                g = i;
            } else if (i < 32) {
                f = (d & b) | ((~d) & c);
                g = (5*i + 1) % 16;
            } else if (i < 48) {
                f = b ^ c ^ d;
                g = (3*i + 5) % 16;          
            } else {
                f = c ^ (b | (~d));
                g = (7*i) % 16;
            }

#ifdef ROUNDS
            if(TEST) printf("f=%x g=%d w[g]=%x\n", f, g, w[g]);
#endif 
            unsigned int temp = d;
            d = c;
            c = b;
            if(TEST) printf("rotateLeft(%x + %x + %x + %x, %d)\n", a, f, k[i], w[g], r[i]);
            b = b + LEFTROTATE((a + f + k[i] + w[g]), r[i]);
            a = temp;
        }
 
        h0 += a;
        h1 += b;
        h2 += c;
        h3 += d;
    }
 
    free(msg);
 
}
 

//use this function to produce hash
//input:
//     string
//output:
//      md5 hash
//      short numeric hash
int md5lib(const char*string_in, char*hash_out, unsigned int*nhash) {
 
    char a[20],b[20],c[20],d[20];
    unsigned char *p;
 
    size_t len = strlen(string_in);

    strcpy(hash_out, "null");

    unsigned char*msg_mod = (unsigned  char*)malloc((int)len*sizeof(unsigned char));
    if(msg_mod==NULL) return -1;

    memcpy(msg_mod, string_in, len+1);

    md5(msg_mod, len);

    free(msg_mod);

    //convert into string
    p=(unsigned char *)&h0;
    sprintf(a, "%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3]);
 
    p=(unsigned char *)&h1;
    sprintf(b, "%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3]);

    p=(unsigned char *)&h2;
    sprintf(c, "%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3]);

    p=(unsigned char *)&h3;
    sprintf(d, "%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3]);

    if(TEST) printf("%s%s%s%s\n", a,b,c,d);
    
    strcpy(hash_out, a);
    strcat(hash_out, b);
    strcat(hash_out, c);
    strcat(hash_out, d);

    //convert into hex (short numeric hash)
    int numa; // = (int)strtol(a, NULL, 16);
    int numb; // = (int)strtol(b, NULL, 16);
    int numc; // = (int)strtol(c, NULL, 16);
    int numd; // = (int)strtol(d, NULL, 16);
    sscanf(a, "%x", &numa);
    sscanf(b, "%x", &numb);
    sscanf(c, "%x", &numc);
    sscanf(d, "%x", &numd);
 
    //numeric unsigned int hash
    *nhash = (unsigned int)(numa^numb^numc^numd);
    
    return 0;
}

//use this function to produce hash
//input:
//     string
//output:
//      md5 hash
//      short numeric hash
int md5cnt(const char*string_in, int lenstring, 
            char*hash_out, unsigned int*nhash) {
 
    char a[20],b[20],c[20],d[20];
    unsigned char *p;
 
    size_t len = (size_t)lenstring;

    strcpy(hash_out, "null");

    unsigned char*msg_mod = (unsigned  char*)malloc((int)len*sizeof(unsigned char));
    if(msg_mod==NULL) return -1;

    memcpy(msg_mod, string_in, len);

    md5(msg_mod, len);

    free(msg_mod);

    //convert into string
    p=(unsigned char *)&h0;
    sprintf(a, "%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3]);
 
    p=(unsigned char *)&h1;
    sprintf(b, "%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3]);

    p=(unsigned char *)&h2;
    sprintf(c, "%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3]);

    p=(unsigned char *)&h3;
    sprintf(d, "%2.2x%2.2x%2.2x%2.2x", p[0], p[1], p[2], p[3]);

    if(TEST) printf("%s%s%s%s\n", a,b,c,d);
    
    strcpy(hash_out, a);
    strcat(hash_out, b);
    strcat(hash_out, c);
    strcat(hash_out, d);

    //convert into hex (short numeric hash)
    int numa; // = (int)strtol(a, NULL, 16);
    int numb; // = (int)strtol(b, NULL, 16);
    int numc; // = (int)strtol(c, NULL, 16);
    int numd; // = (int)strtol(d, NULL, 16);
    sscanf(a, "%x", &numa);
    sscanf(b, "%x", &numb);
    sscanf(c, "%x", &numc);
    sscanf(d, "%x", &numd);
 
    //numeric unsigned int hash
    *nhash = (unsigned int)(numa^numb^numc^numd);
    
    return 0;
}



/*
//--- TEST ---
int main(int argc, char**argv) {

   char hash[40];
   unsigned int nhash;
   
   if (argc < 2) {
      printf("use: %s 'string'\n", argv[0]);
      return 0;
   }

   md5lib(argv[1], hash, &nhash);

   printf("md5: %s\t%u\n", hash, nhash); 

   return 0;
}*/
