///////////////////////////////////////////////////////////////////////////////
//			Liniowy generator liczb losowych
//			           ver. 0.2
//                                (JPW 2018)
//
//
//  Generator liczb losowych typu TRUE PSEUDO RANDOM.
//  Generator korzysta z dwoch zrodel entropi (SEED).
//  1. Generatora sprzetowego Intel RDRAND (*)
//  2. Niemonotonicznej funkcji czasu (rodzielczosc nanosekundy)
//  (*) Intel wprowadzil funkcje RDRAND do CPU od III Generacji procesorow
//      i3/i5/i7 ("Ivy Bridge").
//      AMD dopiero w czerwcu 2015 roku. Z tego powodu w starszych PC
//      to zrodlo entropi (SEED) jest utomatycznie wylaczone. 
//
//  
//                             
//  kompilacja:
//             gcc -Wall prandom_gen.c -mrdrnd
//             
///////////////////////////////////////////////////////////////////////////////     


char*gen_ver = "0.2 (linear) for Linux 64-bit";


#include <stdio.h>
#include <limits.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <cpuid.h>
#include <errno.h>


#define MAX_SEED 1000

unsigned long long init_seed_prnd;



///////////////////////////////////////////////////////////////////////////////
//                         --- PROTOYPY FUNKCJI ---
///////////////////////////////////////////////////////////////////////////////
//funkcja geneorwania bloku danych
//input:
//     buf     - bufor na probki random o wrtosciach (0...255)
//     lenbuf  - liczba probek tzn. romiar bufora buf
//zwraca:
//      -1 - nie udalo sie wygenerowac bloku random w buf
//       0 - blok zostal wygenerowany porawnie
int random_block(unsigned char*buf, int lenbuf);


//wersja generatora
void random_genenerator_version(void) {
     printf(" VERSION: %s\n", gen_ver);
}
///////////////////////////////////////////////////////////////////////////////
//                         --- END PROTOYPY FUNKCJI ---
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//------------------- mnozenie i mod_prndulo duzych liczb integer ------------------
///////////////////////////////////////////////////////////////////////////////
//--- mnozy dwie bardzo dlugie liczby calkowite limit MAX_SEED ---
char*multiply_prnd(char a[],char b[]){

   static char mul[MAX_SEED];
   char c[MAX_SEED];
   char temp[MAX_SEED];
   int la,lb;
   int i,j,k=0,x=0,y;
   long int r=0;
   long sum = 0;

   la=strlen(a)-1;
   lb=strlen(b)-1;
   for(i=0;i<=la;i++) {
      a[i] = a[i] - 48;
   }
   for(i=0;i<=lb;i++) {
      b[i] = b[i] - 48;
   }

   for(i=lb;i>=0;i--) {
      r=0;
      for(j=la;j>=0;j--) {
         temp[k++] = (b[i]*a[j] + r)%10;
         r = (b[i]*a[j]+r)/10;
      }
      temp[k++] = r;
	 x++;
      for(y = 0;y<x;y++) {
   	 temp[k++] = 0;
      }
   }

   k=0;
   r=0;
   for(i=0;i<la+lb+2;i++) {
      sum =0;
      y=0;
      for(j=1;j<=lb+1;j++) {
  	 if(i <= la+j) {
	     sum = sum + temp[y+i];
 	 }
	 y += j + la + 1;
      }
      c[k++] = (sum+r) %10;
      r = (sum+r)/10;
   }

   c[k] = r;
   j=0;
   for(i=k-1;i>=0;i--) {
      mul[j++]=c[i] + 48;
   }

   mul[j]='\0';
   return mul;
}


// --- oblicza mod_prndulo dla very long  numbers limit (MAX_SEED) ---
unsigned long long mod_prnd(char*num, unsigned long long n) {
   if(n==0) return -1;
   int i;
   unsigned long long remainder = 0;
   for (i = 0; num[i] != '\0'; ++i)
      remainder = (remainder * 10 + num[i] - '0') % n;
   return remainder;
}
///////////////////////////////////////////////////////////////////////////////
//------------ END mnozenie i mod_prndulo duzych liczb calkowitych -----------------
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
//                         --- FUNKCJE RANDOMIZE62 ---
///////////////////////////////////////////////////////////////////////////////
// --- ziarno (seed) generatora probek losowych ---
void long_srand(unsigned long long seed){
   init_seed_prnd = seed;
}

unsigned long long m62;
//--- oblicza parametr globalny:  m62 ---
unsigned long long nu_m62(void) {
   int i;
   unsigned long long z = 1L;
   for(i=0; i<62; i++) z = z*2;
   return z-1;
 }

//--- generator losowych probek ---
unsigned long long rng_rand() {
   char result[MAX_SEED];
   char sinit[80];
   char tmp[80] = "16807";

   sprintf(sinit,"%llu",init_seed_prnd);
   strcpy(result,sinit);

   strcpy(tmp,sinit);
   strcpy(result,multiply_prnd(result,tmp));
   
   unsigned long long n = m62;

   init_seed_prnd = mod_prnd(result, n);

   return init_seed_prnd;
}
///////////////////////////////////////////////////////////////////////////////
//                       --- END FUNKCJE RANDOMIZE62 ---
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                    --- SOURCE OF ENTROPY RNRAND ---
/////////////////////////////////////////////////////////////////////////////// 
 /*
  funkcje uruchamiajace RDRAND (Intel/AMD)
  unsigned int __builtin_ia32_rdrand16_step (unsigned short *)
  unsigned int __builtin_ia32_rdrand32_step (unsigned int *)
  unsigned int __builtin_ia32_rdrand64_step (unsigned long long *)
*/


//sprawdza w cpu (intel/AMD) obecnosc funkcji RDRAND
//uwaga, funkcja RNRAND zostala wprowadzona do i3/i5/i7
//dopiero od III generacji "Ivy Bridge". 
//W przypadku AMD dopiero w czerwcu 2015.
int check_cpu(void) {
   const unsigned int flag_RDRAND = (1 << 30);
   unsigned int eax, ebx, ecx, edx;
  //nalezy ustawić rejestr eax=1 aby uzyskac info w trybie extended
  //zwraca status rdrand w ecx na pozycji 30 bitu 
   __get_cpuid(1, &eax, &ebx, &ecx, &edx);
   return (int)((ecx & flag_RDRAND) == flag_RDRAND);

} 


//gnerator ziarna -  pobiera z RDRAND (pierwsze zrodlo entropi) 
int randoms3(unsigned long long *rnd) {
    int retries= 10;
    unsigned long long rand64;

    while(retries--) {
        if ( __builtin_ia32_rdrand64_step(&rand64) ) {
            *rnd = rand64;
            return 1;
        }
    }
    return 0;
}


//--- zrodlo entropii RDRAND ---
//generuje seed w przypadku RNRAND (Intel/AMD)
//zwraca -1 jesli error
//zwraca 0 jesli wygenerowal seed
int seed_RNRAND(unsigned long long *seed) {

   //sprawdza obecnosc RNRAND w procesorze
   if(!check_cpu()) return -1;

   //tworzy seed_RNRAND
   if(!randoms3(seed)) return -1;

   return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                         --- END SEED FROM RNRAND ---
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
// 				--- SEED FROM TIMER ---
///////////////////////////////////////////////////////////////////////////////
// --- zrodlo entropi (timer nasekundowy) ---
//niemonotoniczna funkcja czasu dokładnosc nansekundy
int nano_timer(unsigned long long *tms) {

   char sec[40], nsec[40];
   char stime[80];
   long long ns;
   int i;
   struct timespec res;

   char tmp[80] = "16807";
 
   if(clock_gettime(CLOCK_REALTIME, &res) < 0) return -1;

   sprintf(sec, "%ld", (long)res.tv_sec);
   sprintf(nsec, "%ld", res.tv_nsec);
   strcpy(stime, sec);
   strcat(stime, nsec);

   strcpy(stime, multiply_prnd(tmp, stime));

   ns = 1L;
   for(i=0; i<62; i++) ns = ns*2;
   ns = ns-1;

   *tms = mod_prnd(stime, ns);

   return 0;
}
///////////////////////////////////////////////////////////////////////////////
// 				--- SEED FROM TIMER ---
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
//				      --- SEED ---
///////////////////////////////////////////////////////////////////////////////

//wyznaczenie ziarna entorpi dla generatora PRAND
int seed_entropy(unsigned long long *seed) {

  unsigned long long l_rdrnd;
  unsigned long long l_timer;
  unsigned long long l_timer1;

  char a[200], b[40];
 
  //pobiera entropy z RDRAND (Intel)
  int flg_rnrand = seed_RNRAND(&l_rdrnd);

  //pobiera seed 
  int flg_timer = nano_timer(&l_timer);
  if(flg_timer <0 ) return -1; //nie da sie użyć seed_entropy
  
  //czeka az zmieni sie czas w nanosekundach
  if(nano_timer(&l_timer1) < 0) return -1;
  while(l_timer == l_timer1) nano_timer(&l_timer);

  //wrowadzamy przypadkowe opoznienie
  struct timespec req = { 
                         .tv_sec = 0, 
                         .tv_nsec = (long)(l_timer % 501) 
                         };
  struct timespec rem;

//male losowe opoznienie (o..501) ns dla timera 
label_retry:
   errno=0;
   int ret = nanosleep(&req, &rem);
   if(ret) {
      if(errno == EINTR) {
         req.tv_sec = rem.tv_sec;
         req.tv_nsec = rem.tv_nsec;
         goto label_retry;
      }
   }

   if(nano_timer(&l_timer) < 0) return -1;

   unsigned long long ns = 1L;
   int i; for(i=0; i<62; i++) ns = ns*2;
   ns = ns-1;

   //kombinacja dwoch zrodel entrpi (RDRAND z niemononiczna funkcja czasu)
   if(flg_rnrand == 0) {
      sprintf(a, "%llu", l_rdrnd);
      sprintf(b, "%llu", l_timer);
      strcpy(a, multiply_prnd(a,b));
      *seed = mod_prnd(a, ns);
      return 0;
   }

   //jesli RDRAND nie jest obecny w CPU, korzysta tylki z jendgo
   //zrodla entropi
   *seed = l_timer % ns;
   return 0;
}

///////////////////////////////////////////////////////////////////////////////
//				    --- END SEED ---
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
//                           --- block of random numbers ---
///////////////////////////////////////////////////////////////////////////////
//generuje dane randomize ze zbioru (0..255) do buf
//najlepiej uzyc buf o min. romizarze 4KB, raczej unikac maniejszych
//buforow na liczby random
int random_block(unsigned char*buf, int lenbuf) {

   int i,j;
   unsigned long long seed;
   m62 = nu_m62(); 

   //inicjuje ziarno
   if(seed_entropy(&seed) < 0) return -1;
   long_srand(seed);

   int t= lenbuf/2;

   for(i=0; i<t; i++) {
      j = i*2; 
      *(buf+j) = (char)(rng_rand() % 256);    
   }

   if(seed_entropy(&seed) < 0) return -1;
   long_srand(seed);

   for(i=0; i<t; i++) {
      j = i*2 + 1; 
      *(buf+j) = (unsigned char)(rng_rand() % 256);    
   }

   return 0; 
}
///////////////////////////////////////////////////////////////////////////////
//                       ---  END block of random numbers ---
///////////////////////////////////////////////////////////////////////////////
