///////////////////////////////////////////////////////////////////////////////
//
//                               ARCHI
//			              Funkcje dla executorUIDC.c
//			                    mserverUIDC.c
//                             (library)
//	                             ver 0.2
//                            (JPW - 2019)
//
//
///////////////////////////////////////////////////////////////////////////////



#include <stdio.h>
#include <string.h>
#include <time.h>
#ifndef SELECT 
#define SELECT
#include "SRC/select_value.c" 
#endif


#ifndef READ_CONFIG 
#define READ_CONFIG
#include "SRC/read_config.c"
#endif


#include "SRC/gen_UID_lib.c"
#include "exec_functions_additional.c"




///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// ----------------------------- prototypy funkcji ----------------------------
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

// --- kod startowy ---
int start_code(const char*in, char*out, const char*ip_client); //dla admina
int chk_start_code(const char*in, char*out); //sprawdzanie kodu startowego



// --- tworzy UID32&EUID lub UIDP&EUIDP ---
int get_UID64(char*reply);



// --- tworzy specjalne konto PESEL ---
int get_special_pesel(const char*in, char*out);



// --- dodawanie i zmiana danych node (department w LAN) ---
// (wszystkie UID32, EUID sa takie same)
// dziala z poziomu amina UIDL
int add_department(const char*in, char*out);

// --- blokowanie NODE (departamentu w LAN) ---
int block_department(const char*in, char*out); 

// --- pobieranie danych NODE ---
int get_dep_data(const char*in, char*out);



// --- dodawanie uzytkownika ---
int add_usr(const char*in, char*out);

// --- sprawdzanie uzytkownika ---
int check_user(const char*in, char*out);

//--- blokowanie uzytkownika z poziomu admkina UIDC ---
////<<< ---- moze wprowadzic ip_client (?) 
int block_user_by_central_admin(const char*in, char*out);

// --- blokowanie uzytkownika z poziomu serwera UIDL ---
int block_user(const char*in, char*out);



//--- rejestracja konta PESEL OBJ (obiektu) ---
//tylko dla numerycznego, Nx, RNx, RSx(specjalne) gdzie x to same cyfry
int add_pesel_obj(const char*in, char*out);



// --- MAPA kont PESEL ---
// --- pobieranie liczby wpisow danego konta (rejestracji na NODAch ---
int get_nlines_pesel(const char*in, char*out);

// --- MAPA kont PESEL ---
//--- pobiera wybrana linie z mapy kont PESEL ---
int get_line_pesel_obj(const char*in, char*out);

// --- MAPA kont PESEL ---
// --- pobniera kompletne dane NODE (wraz z IP WAN) ---
int get_node(const char*in, char*out);


// --- Dodawanie lub aktualizacja IP NODE (WAN) ---
int add_ip_node(const char*in, char*out);


// --- INFO o tym serwerze (VERSION) ---
int versionUIDC(const char*in, char*out);



// --- RESTORE BACKUP ---
int restore_files_local(const char*in, char*out, const char*ip_client);



// --- pobiera port z pliku config ---
int get_uidc_config(const char*in, char*out);

// --- zmienia port server UIDC ---
int chg_uidc_config(const char*in, char*out, const char*ip_client);



///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//--------------------------Koniec prototypow funkcji -------------------------
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////







///////////////////////////////////////////////////////////////////////////////
//                             Tworzenie kodu START 
///////////////////////////////////////////////////////////////////////////////

//------------------------------- Pomocnicze START ----------------------------

//sprawdza date waznosci kodu startowego
//format wejsciowy sdate: 20170130-h:m:s
//data graniczna ldate: 20180431
int valid_date( const char*vdate) {

	char cdate[40];

   if(get_time(cdate) < 0) return -1;

	char*p = strchr(cdate, '-');
	if(p !=NULL) *p = '\0';

	if(atoi(cdate)	<= atoi(vdate)) return 1;
	return 0;
		
} 


//sprawdza czy linia w pliku jest zgodna ze stringem
//jeli vdate=null to nie sprawdza daty waznsci i start code jest zawsze wazny
int check_line_in_file(const char*path, const char*str, const char*vdate) {

	char buf[256];
	char edate[80];
	char *p;

	FILE*fd = fopen(path, "r");
	if(fd == NULL) return -1;

	while(fgets(buf, 256, fd) != NULL) {
	    p = strstr(buf, str);
	    if(p != NULL) {
		if(strcmp(vdate, "null") != 0) {
		   select_value(buf, "VDATA", edate);
		   if(strcmp(edate, "null") == 0) {fclose(fd); return 0;} //nie sprawdza daty 
		   if(strcmp(edate, "NULL") == 0) {fclose(fd); return 0;} //tez nie sprawdza daty
		   if( valid_date(edate) < 1) {
		   	fclose(fd);
		   	return -1;
		   }
		}
		fclose(fd);
		return 0;
	    }
	} 
	fclose(fd);
	return -1;
}
//----------------------------- END Pomocznicze START -------------------------



//-------------- dla administratora systemu centralnego (START) ---------------

// --- admin tworzy i pobiera code startowy rejestracji nowego UIDL --- 
//(kod rejstracyjny pobiera admin UIDC)
//OP=get_start_code&LDATE=20180812&
//RE=get_start_code&CODE=...& //lub ...CODE=null& 
//Kod rejestracyjny znajduje sie w pliku START/A/B/C/SCODE
int start_code(const char*in, char*out, const char*ip_client) {

	char st[80];
	char abc[80];
	char path[128];
	char *p;

	//0. Sprawdza czy polecenie jest poprawne
	p = strstr(in, "get_start_code");
	if(p == NULL) {
	   strcpy(out, "RE=get_start_code&");
	   strcat(out, "CODE=null&");
	   return -1;
	}

	//sprawdza czy client znajduje sie w sieci lokalnej
	if( filtr_lan_ip(ip_client, "null") )  {

		goto label_start;

	   } else {
	   	strcpy(out, "RE=get_start_code&");
	   	strcat(out, "CODE=null&");
	   	return -1;
	   }

label_start:

	//1. Tworzy startowy kod
	if(start_CODE(st) < 0) {
	   strcpy(out, "RE=get_start_code&");
	   strcat(out, "CODE=null&");
	   return -1;
	}
	//sprawdza czy kod startowy juz nie wystepuje w pliku
	st_abc(st, abc);
	strcpy(path, start_base); 
	strcat(path, abc);
	strcat(path, SCODE); 
	if(check_line_in_file(path, st, "null") == 0) goto label_start;

	//2. Zapisuje do START (bazy)
	st_abc(st, abc);
	strcpy(path, start_base); 
	strcat(path, abc);
	strcpy(abc, "CODE=");
	strcat(abc, st);
	strcat(abc, "&");
	//dodaje date waznosci kodu rejestracyjnego
	//po tej dacie kod moze byc niewazny i trzeba 
	//bedzie utworzyc nowy kod rejestracyjny
	select_value(in, "VDATE", st);
	if(strcmp(st, "NULL") != 0) {
		strcat(abc, "VDATE="); 
		strcat(abc, st);
		strcat(abc, "&");		
	}
	strcpy(st, abc);
	if(add_text_line( path, SCODE, st) < 0) {
	   strcpy(out, "RE=get_start_code&");
	   strcat(out, "CODE=null&");
	   return -1;
	}
	
	//<<<<<<<<<<<<<  jeszcze zapis do list_start_base.q (backup)
	//CODE=...&VDATE=..& 
   strcpy(path, list_base);
	strcat(path, "/");
	strcat(path, LIST_START);
	add_text_line( path, SCODE, st);

	//4. Zwraca adminowi kod rejestracyny dla node
	strcpy(out, "RE=get_start_code&");
	strcat(out, st);

	return 0;
}
//--------------- END dla tylko dla admina centralnego (START) ----------------



//------------------ SPRAWDZA  CZY START CODE JEST ZGODNY ---------------------
		
//sprawdza czy start code jest poprawny
//OP=chk_scode&CODE=...& //code przeslny przy rejestracji
//RE=chk_scode&STAT=done& // jesli brak zgodnosci STAT=null&
int chk_start_code(const char*in, char*out) {

	char *p;
	char abc[80];
	char st[80];
	char path[80];
	
	//sprawdza czy jest to poprawana instrukcja
	p = strstr(in, "chk_scode");
	if(p == NULL) {
	  strcpy(out, "RE=chk_scode&STAT=null&");
	  return -1;
	}

	//rzutowanie start-code
	select_value(in, "CODE", st);
	if(strcmp(st, "NULL") == 0) {
	  strcpy(out, "RE=chk_scode&STAT=null&");
	  return -1;
	}

	st_abc(st, abc);
	strcpy(path, start_base); 
	strcat(path, abc);
	strcat(path, SCODE); 

	if(check_line_in_file(path, st,"CHECK-VALID-DATA") < 0) {
	  strcpy(out, "RE=chk_scode&STAT=null&");
	  return -1;	
	}
	
	strcpy(out, "RE=chk_scode&STAT=done&");
	return 0;
}

//-------------------- END SPRAWDZA  CZY START CODE JEST ZGODNY ---------------

///////////////////////////////////////////////////////////////////////////////
//                             END Tworzenie kodu START 
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//------------------------------- Generowanie UID64 ---------------------------
int get_UID64(char*reply) {
	char uid[35],euid[35];

	if(gen_UID64T(uid, euid, 0) == -1) {
		strcpy(reply,"RE=create_uid64&STATUS=refused&");
		return -1;
		}
	strcpy(reply,"OP=get_UID64&UID32=");strcat(reply,uid);strcat(reply,"&");
	strcat(reply,"EUID=");strcat(reply,euid);strcat(reply,"&");
	return 0;
}
//----------------------------- END Generowanie UID64 -------------------------
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//------------------------- Tworzenie PESEL Specjalnego -----------------------
///////////////////////////////////////////////////////////////////////////////
//tworzy konto specjalne RS, ktore jest rejestrowane w UIDC
//podobnie jak konta numerczyne PESEL N, RN
//OP=get_special_pesel&
//Zwraca: 
//   RE=get_special_pesel&STATUS=done&PESEL=RS000000103&
//   lub w przypadku bledu
//   RE=get_special_pesel&STATUS=refused&CP=<powod bledu>&

int get_special_pesel(const char*in, char*out) {

	char st[80] = "null";
	char path[40];
	char *p;
	char pesel[15];

	FILE*fp = popen("BIN/numerator", "r");
	if(fp == NULL) {
	  strcpy(out, "RE=get_special_pesel&STATUS=refused&CP=no_file[1]&");
	  return -1;	
	}

	fgets(st, 20, fp);
	pclose(fp);

	p = strchr(st, '\n');
	if(p != NULL) *p = '\0';

	if(strcmp(st, "null") == 0) {
	  strcpy(out, "RE=get_special_pesel&STATUS=refused&CP=no_reply&");
	  return -1;	
	}
   
	strcpy(pesel, st);

   // backup (cp DISK/rs_last_number.txt  DISK/LIST/rs_last_number.txt)
	strcpy(path, list_base);
	p = strchr(path, '/');
	if(p != NULL) *p = '\0';
	strcpy(st, "cp ");
	strcat(st, path);
	strcat(st, "/");
	strcat(st, LAST_NUMERATOR);
	strcat(st, " ");
	strcat(st, list_base);
	strcat(st, "/");
	strcat(st, LAST_NUMERATOR);
	fp = popen(st, "r");
	if(fp == NULL) {
	  strcpy(out, "RE=get_special_pesel&STATUS=refused&CP=no_copy[1]&");
	  return -1;	
	}
	*st = '\0';
	fgets(st, 80, fp);
	pclose(fp);

	if(strlen(st) > 3) {
	  strcpy(out, "RE=get_special_pesel&STATUS=refused&CP=no_copy[2]&");
	  return -1;	
	}

	strcpy(out, "RE=get_special_pesel&STATUS=done&PESEL=");
	strcat(out, pesel);
	strcat(out, "&");

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//---------------------- END Tworzenie PESEL Specjalnego ----------------------
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                                DEPARTAAMENT
///////////////////////////////////////////////////////////////////////////////        
// rejestracja departamentu
//OP=add_department&UID32=null&EUID=null&NAME=..&DEP=...
//   &NIP=..&REGON=...&LIP=...&PUID=...&PORTP=..&PORTQ=...&NODES=...&
//Zwraca:
//RE=register_node&STATUS=done&UID32=..&EUID=..&
//lub
//RE=register_node&STATUS=refused&
int add_department(const char*in, char*out) {

	int ret;
	char st[512];

	char uid32[35];
	char euid[35];
	char dep[15];
	char name[80];
	char nip[15];
	char regon[15];
	char puid[10];
	char portp[10];
	char portq[10];
	char nodes[6];

printf("IN: %s\n", in);

	//ropzkowuje i parsuje sting "in"
	ret = strip_dpt_string(in,
			uid32, euid, name,
			dep, nip, regon, 
			puid, portp, portq, nodes); 

	if(!ret) {
	  strcpy(out, "RE=add_department&STATUS=refused&CP=incorrect_qvector&");
	  return -1;	
	}

	//sprawdza czy przy NIP/REGON jest juz zarejestrowany DEP
	ret = check_dep_in_pesel_base(dep, nip, regon, uid32, euid);
	if(ret == 0) {
		//jesli 0 to nie mozna zmienic argumentow
		//musi byc NIP=..&DEP=..& zgodne lub zgodne REGON=..&DEP=...&NIP=null&
		// poprzednio zapisanymi tzn. 
		// mozna usupelnic nip lub regon (jesli jeden z nich jest null)
		// ale nie mozna zmienic dep
		strcpy(out, "RE=add_department&STATUS=refused&CP=incorrect_arguments&");
		return -1;
	}
	
	//--- rejestruje przeslane dane w strukturach i plikach backup ---
	
	//tworzy UID32, EUID
	if(strcmp(uid32, "null") == 0 && strcmp(euid, "null") == 0) {
		if(gen_UID64T(uid32, euid, 0) == -1) {
	  		strcpy(out, "RE=add_department&STATUS=refused&CP=no_gen_UID&");
	  		return -1;	
		}
	}

	//sklada qvector	
	create_dep_string(uid32, euid, name, dep, nip, regon, st);


	//zapis do struktury PESEL (NIP & REGON)
	//UID32=..&EUID=...&NAME=..&DEP=..&NIP=..&REGON=..&DATE=..&
	if(strcmp(nip, "null") != 0) {
		ret = add_node_to_pesel(nip, st);	//NIP
		if(ret < 0) {
	  		strcpy(out, "RE=add_department&STATUS=refused&CP=no_file[1]&");
	  		return -1;	
		}
	}
	if(strcmp(regon, "null") != 0) {
		ret = add_node_to_pesel(regon, st);	//REGON
		if(ret < 0) {
	  		strcpy(out, "RE=add_department&STATUS=refused&CP=no_file[2]&");
	  		return -1;	
		}
	}

	//zapis do LIST_PESEL_NOD (backup)
	add_text_line(list_base, LIST_PESEL_NOD, st);

	//zapis do strutury UID32BASE
	//postac qstring:
	//UID32=..&EUID=...&DEP=..&PUID=..&PORTP=..&PORTQ=..&NODES=..&DATE=..&
	create_dep_string_uid32base(uid32, euid, dep, 
										 puid, portp, portq, nodes, st); 

	//zapis do UID32BASE
	ret = add_node_to_uid32base(uid32, st);
		if(ret < 0) {
	  		strcpy(out, "RE=add_department&STATUS=refused&CP=no_file[3]&");
	  		return -1;	
		}

	//zapis do LIST_UID32BASE (backup)
	add_text_line(list_base, LIST_UID32BASE, st);

	//wysyla odpowiedz do UIDL
	create_dep_string_full(uid32, euid, name, dep, nip, regon, 
	puid, portp, portq, nodes, st);
	strcpy(out, "RE=add_department&STATUS=done&");
	strcat(out, st);

	return 0;
		
}                        



//blokowanie departamentu (z poziomu UIDC, obsluga przez admina)
//(tylko dla admina)
//OP=block_department&DEP=null&NIP=...&REGON=...&  - blokuje wszystke dep.
//OP=block_department&DEP=...&NIP=...&REGON=...&   - blokuje wybrany dep.
//Zwraca:
//RE=block_department&STATUS=done&
//lub
//RE=block_department&STATUS=refused&CP=<powod odmowy>&
int block_department(const char*in, char*out) {

	int ret;
	char st[512];

	char uid32[35];
	char euid[35];
	char dep[15];
	char nip[15];
	char regon[15];

	//rozkladanie "in"
	ret = strip_small_dpt_string(in, nip, regon, dep);
	if(!ret) {
	  		strcpy(out, 
				"RE=block_department&STATUS=refused&CP=incorrect_qvector&");
	  		return -1;	
	}

	//tworzenie nowego euid
	if(gen_UID64T(st, euid, 0) < 0) {
	  	strcpy(out, "RE=block_department&STATUS=refused&CP=no_gen_UID&");
	  	return -1;	
	}


	//pobiera dane ze struktury PESEL base dla danego dep&pesel
	if(strcmp(nip, "null") != 0) {
		//zmiana euid w bazie PESEL
    	ret = chg_euid_in_pesel_base(dep, nip, regon, euid, uid32);
	 	if(ret < 0) {
	  		strcpy(out, 
				"RE=block_department&STATUS=refused&CP=no_file[1]&");
	  		return -1;	
		}

		//zmiana euid w bazie UID32BASE
		ret = chg_euid_in_uid32base(dep, euid, uid32);
		if(ret < 0) {
			strcpy(out,
				"RE=block_department&STATUS=refused&CP=no_file[2]&");
	  		return -1;				
		}
	} 
	else {
		if(strcmp(regon, "null") != 0) {
			//zmiana euid w bazie PESEL
    		ret = chg_euid_in_pesel_base(dep, nip, regon, euid, uid32);
	 		if(ret < 0) {
	  			strcpy(out, 
					"RE=block_department&STATUS=refused&CP=no_file[3]&");
	  			return -1;	
			}
			//zmiana euid w bazie UID32BASE
			ret = chg_euid_in_uid32base(dep, euid, uid32);
			if(ret < 0) {
				strcpy(out,
				"RE=block_department&STATUS=refused&CP=no_file[4]&");
	  		return -1;				
			}
		}	else {
	  			strcpy(out, "RE=block_department&STATUS=refused&CP=no_gen_UID&");
	  		return -1;	
			}
	} 
	
	strcpy(out, "RE=block_department&STATUS=done&");

	return 0;
}



//pobieranie danych departamentu przez UID32
//OP=get_dep_data&DEP=..&UID32=...&
//Zwraca:
//   RE=get_dep_data&STATUS=done&UID32=...&EUID=..&DEP=...
//   &LIP=...&PUID=...&PORTP=..&PORTQ=...&NODES=..
//   &DATE=..&
// lub
// RE=get_dep_data&STATUS=refused&CP=<powod odmowy>& 
int get_dep_data(const char*in, char*out) {

		char dep[15];
		char uid32[35];
		char qstr[512];
		int ret;

		ret = 1;
		select_value(in, "DEP", dep);
		if(strcmp(dep, "NULL") == 0) ret = 0;
		select_value(in, "UID32", uid32);
		if(strcmp(uid32, "NULL") == 0) ret = 0;

		if(!ret) {
			strcpy(out, 
				"RE=get_dep_data&STATUS=refused&CP=incorrect_qvector&");
			return -1;
		}

		ret = read_dep_data_in_UID32BASE(dep, uid32, qstr);
		if(ret < 0) {
			strcpy(out, 
				"RE=get_dep_data&STATUS=refused&CP=no_file&");
			return -1;
		}

		strcpy(out, "RE=get_dep_data&STATUS=done&");
		strcat(out, qstr);

		return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                               END DEPARTAAMENT
/////////////////////////////////////////////////////////////////////////////// 




///////////////////////////////////////////////////////////////////////////////
//                               USER REGISTRATION
///////////////////////////////////////////////////////////////////////////////
	// dodaje  admina/usera - "admin registration" 
	//    lub zmienia STAT=<admin/user>&
	// OP=add_usr&UID32=..&EUID=..&UIDP=null&EUIDP=null
	// &NAME=<imie i nazwisko>&PESEL=...&DEP=...&STAT=admin&
	// nie zapisuje w UIDC RANGE oraz ACT (tylko w UIDL)
	// dopisuje: RANGE=1111111&ACT=A&
	//
	// Odpowiedz:
	//   RE=add_usr&STATUS=done&UID32=..&EUID=..&UIDP=...&EUIDP=...
	//   &NAME=<imie i nazwisko>&
   //   PESEL=...&DEP=...&STAT=admin&RANGE=...&ACT=A&DATE=..&
   //   lub
   //   RE=add_usr&STATUS=refused&CP=<powod odmowy>&
int add_usr(const char*in, char*out) {

	int ret;
	struct userfull usr;
	char qstr[512];
	char statt[10];

	//parsowanie i rozlozenie qvectora wejsciowego (sprawdza form. pesel)
	ret = strip_user_string(in, 
		usr.uid32, usr.euid, usr.uidp, usr.euidp, usr.pesel,
		usr.dep, usr.name, usr.statt, usr.range, usr.act);
	if(!ret) {
		strcpy(out, "RE=add_usr&STATUS=refused&CP=incorrect_qvector&");
		return -1;
	}

	//sprawdza obenosc wpisu w bazie PESEL
	ret = check_user_pesel(usr.pesel, qstr);
	if(ret == -2) {
		strcpy(out, "RE=add_usr&STATUS=refused&CP=incorrect_pesel&");
		return -1;
	}
	if(ret == -1) goto label_new_user;

	strcat(qstr, "RANGE=");
	strcat(qstr, usr.range);
	strcat(qstr, "&ACT=");
	strcat(qstr, usr.act);
	strcat(qstr, "&");


	if(ret == 0) {
		//sprawdza czy nie bylo zmiany STAT z admin->user lub user->admin
		select_value(qstr, "STAT", statt);
		if(strcmp(statt, usr.statt) != 0 
			&& strcmp(statt,"NULL") != 0) { 
 				strip_user_string(qstr, 
					usr.uid32, usr.euid, usr.uidp, usr.euidp, usr.pesel,
					usr.dep, usr.name, statt, usr.range, usr.act);
				goto label_old_user; //zamienia warosc STAT i wpisuje do struktur
		}
		else {
			strcpy(out, "RE=add_usr&STATUS=done&");
			strcat(out, qstr);
			return 0;
		}
	}

label_new_user:

	//generuje UIDP oraz EUIDP dla nowego uzytkownika
	if(gen_UID64T(usr.uidp, usr.euidp, 0) == -1) {
		strcpy(out, "RE=add_usr&STATUS=refused&gen_error&");
		return -1;
		}
	
label_old_user:

	//tworzy string do zapisu
	if(strcmp(usr.statt, "admin") == 0) {
			create_user_small_string( usr.uid32, usr.euid, 
				usr.uidp, usr.euidp, usr.pesel,
				usr.dep, usr.name, "admin",  
		   	qstr);
			//zapis do struktur i plikow 
			ret = save_user_to_structures(usr.uidp, usr.pesel, qstr);
			if(ret < 0) {
				strcpy(out, "RE=add_usr&STATUS=refused&CP=no_file[1]&");
				return -1;
			} 
			//odpowidz w przypadku --- admina ---
			create_user_qstring_full( usr.uid32, usr.euid, 
				usr.uidp, usr.euidp, usr.pesel,
				usr.dep, usr.name, "admin", "11111111", "A", qstr);	
			//tworzy qstring odpowiedzi
			strcpy(out, "RE=add_usr&STATUS=done&");
			strcat(out, qstr);
			return 0;	
	} else {
			//odpowidz w przypadku --- user ---
			create_user_qstring_full( usr.uid32, usr.euid, 
				usr.uidp, usr.euidp, usr.pesel,
				usr.dep, usr.name, usr.statt, usr.range, usr.act,  qstr);
			//zapis do struktur i plikow 
			ret = save_user_to_structures(usr.uidp, usr.pesel, qstr);
			if(ret < 0) {
				strcpy(out, "RE=add_usr&STATUS=refused&CP=no_file[2]&");
				return -1;
			} 
			//tworzy qstring odpowiedzi
			strcpy(out, "RE=add_usr&STATUS=done&");
			strcat(out, qstr);
	}

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                              END USER REGISTRATION
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                                   CHECK USER
///////////////////////////////////////////////////////////////////////////////
// --- check user w UID32BASE ---
// OP=check_user&UID32=..&EUID=..&UIDP=..&EUIDP=..&
// Zwraca:
// 	OP=check_user&STATUS=done&
// 	lub
// 	OP=check_user&STATUS=refused&CP=<powod odmowy>&
int check_user(const char*in, char*out) {
	
	char uid32[35];
	char euid[35];
	char uidp[35];
	char euidp[35];
	int ret = 1;

	select_value(in, "UID32", uid32);
	if(strcmp(uid32, "NULL") == 0) ret = 0;
	select_value(in, "EUID", euid);
	if(strcmp(euid, "NULL") == 0) ret = 0;
	select_value(in, "UIDP", uidp);
	if(strcmp(uidp, "NULL") == 0) ret = 0;
	select_value(in, "EUIDP", euidp);
	if(strcmp(euidp, "NULL") == 0) ret = 0;

	if(!ret) {
		strcpy(out, "RE=check_user&STATUS=refused&CP=incorrect_qvector&");
		return -1;
	}

	ret = check_user_in_uid32base(uid32, euid, uidp, euidp);
   if(ret < 0) {
		strcpy(out, "RE=check_user&STATUS=refused&CP=blocked_user&");
		if(ret == -1) strcpy(out, "RE=check_user&STATUS=refused&CP=blocked_node&");
		return -1;
	}

	strcpy(out, "RE=check_user&STATUS=done&");

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                                 END CHECK USER
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                              BLOCK USER FROM UIDC
///////////////////////////////////////////////////////////////////////////////
	//z poziomu admina UIDC
	//nalezy podac PESEL
	//OP=block_user_by_central_admin&PESEL=...& 
	//RE=block_user_by_central_admin&STATUS=done&
	//lub
	//RE=block_user_by_central_admin&STATUS=refused&
int block_user_by_central_admin(const char*in, char*out) {

	char pesel[15];
	char euidp[35];
	char uidp[35];
	char qstr[512];

	struct userfull usr;

	select_value(in, "PESEL", pesel);
	if(strcmp(pesel, "NULL") == 0) {
		strcpy(out, 
		"RE=block_user_by_central_admin&STATUS=refused&CP=incorrect_qvector[1]&");
		return -1;
	}

	//pobiera dane z bazy PESEL
	if(read_user_from_pesel(pesel, qstr) < 0) {
		strcpy(out, 
		"RE=block_user_by_central_admin&STATUS=refused&CP=no_file&");
	}

	//rozklada qstr
	strip_user_string(qstr,
		usr.uid32, usr.euid, usr.uidp, usr.euidp, usr.pesel,
		usr.dep, usr.name, usr.statt, usr.range, usr.act);

	//sprawdza czy pesel w bazie jest taki sam jak w "in"
	if(strcmp(usr.pesel, pesel) != 0) {
		strcpy(out, 
		"RE=block_user_by_central_admin&STATUS=refused&CP=incorrect_qvector[2]&");
		return -1;

	}

	//generuje UIDP oraz EUIDP dla nowego uzytkownika
	if(gen_UID64T(uidp, euidp, 0) == -1) {
		strcpy(out, "RE==block_user_by_central_admin&STATUS=refused&gen_error&");
		return -1;
		}

	//sklada q-vector
	create_user_small_string( usr.uid32, usr.euid, 
			usr.uidp, euidp, usr.pesel,
			usr.dep, usr.name, usr.statt,  
		   qstr);  	

	//zapisuje do struktur 
   if(save_user_to_structures(usr.uidp, pesel, qstr) < 0) {
		strcpy(out, 
		"RE=block_user_by_central_admin&STATUS=refused&CP=no_file&");
		return -1;
	}	


	//-------------------------------------------------------------------------
	//wysyla do UIDL: OP=block_user&PESEL=...&
	if(send_block_usr_to_UIDL(usr.uid32, pesel, euidp) < 0) {
	strcpy(out, 
		"RE=block_user_by_central_admin&STATUS=refused&CP=no_connect_UIDL&");
		return -1;
	}
	//-------------------------------------------------------------------------


	//tworzy qstring odpowiedzi
	strcpy(out, "RE=block_user_by_central_admin&STATUS=done&");

	return  0;
}
///////////////////////////////////////////////////////////////////////////////
//                            END BLOCK USER FROM UIDC
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                        BLOCK USER FROM UIDL (hard block)
///////////////////////////////////////////////////////////////////////////////
	//(z poziomu admina UIDL - ogolenie serwera UIDL)
	//UIDL przsyla juz zmieniony euidp
	//OP=block_user&&UID32=..&EUID=..&UIDP=...&EUIDP=...
	// &NAME=<imie i nazwisko>&PESEL=...&DEP=...&STAT=admin&
	// nie zapisuje w UIDC RANGE oraz ACT (tylko w UIDL)
	// dopisuje: RANGE=1111111&ACT=A&DATE=...&
	//Opowiedz
	//RE=block_user&STATUS=done&
	//lub
	//RE=block_user&STATUS=refused&CP=<powod odmowy>&
int block_user(const char*in, char*out) {

	struct userfull usr;
	char qstr[512];
	int ret;

	//rozklada qvector "in"
	ret =  strip_user_string(in,
		usr.uid32, usr.euid, usr.uidp, usr.euidp, usr.pesel,
		usr.dep, usr.name, usr.statt, usr.range, usr.act);

	if(!ret) {
		strcpy(out, "RE=block_user&STATUS=refused&CP=incorrect_qvector&");
		return -1;
	} 

	//sklada user qvector dla zapisu w UIDC
	create_user_small_string( usr.uid32, usr.euid, 
			usr.uidp, usr.euidp, usr.pesel,
			usr.dep, usr.name, usr.statt,  
		   qstr); 

	//zapisuje do struktur 
   ret = save_user_to_structures(usr.uidp, usr.pesel, qstr);
	if(ret < 0) {
		strcpy(out, "RE=block_user&STATUS=refused&CP=no_file&");
		return -1;
	}	

	strcpy(out, "RE=block_user&STATUS=done&");

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                      END BLOCK USER FROM UIDL (hard block)
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                              ADD PESEL OBJ TO MAP
///////////////////////////////////////////////////////////////////////////////
	// UIDL przesyla jesli utworzone zostanie nowe konto PESEL przez pompe Q/P
	// tylko dla pesel zgodne z formula (124.., N123... RN123.., RS123..)
	// dane dodawane sa do pliku PESEL_OBJ.q 
	// OP=add_pesel_obj&PESEL=..&UID32=..&DEP=..&
	// Zwraca:
	// 	OP=add_pesel_obj&STATUS=done&
	// 	lub w przypadku bledu
	// 	OP=add_pesel_obj&STATUS=refused&
int add_pesel_obj(const char*in, char*out) {

	int ret;
	char uid32[35];
	char pesel[15];
	char dep[15];
	char qstr[128];

	//parsowanie "in"
	ret = 0;
	select_value(in, "PESEL", pesel);
	select_value(in, "UID32", uid32);
	select_value(in, "DEP", dep);
	if(strcmp(pesel, "NULL") == 0)  ret = 1;
	if(strcmp(uid32, "NULL") == 0)  ret = 1;
	if(strcmp(dep,   "NULL") == 0)  ret = 1;

	if(ret) {
		strcpy(out, "RE=add_pesel_obj&STATUS=refused&CP=incorrect_qvector&");
		return -1;
	}

	//sprawdza formule PESEL
   ret = check_form_pesel(pesel);
	if(!ret) {
		strcpy(out, "RE=add_pesel_obj&STATUS=refused&CP=incorrect_pesel&");
		return -1;
	}

	//sklada qstr do zapisu w plikach
	strcpy(qstr, "PESEL=");
	strcat(qstr, pesel);
	strcat(qstr, "&UID32=");
	strcat(qstr, uid32);
	strcat(qstr, "&DEP=");
	strcat(qstr, dep);
	strcat(qstr, "&");

	ret = save_pesel_obj(pesel, qstr);
	if(ret < 0) {
		strcpy(out, "RE=add_pesel_obj&STATUS=refused&CP=no_file&");
		return -1;
	}
	
	strcpy(out, "RE=add_pesel_obj&STATUS=done&");

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                             END ADD PESEL OBJ TO MAP
///////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////
//                               GET DATA FROM MAP
////////////////////////////////////////////////////////////////////////////////
	//pobieranie liczby linii rejestracji danego konta PESEL (tylko dla OBJ)
	//OP=get_nlines_pesel&PESEL=...&
	//Odpowiedz:
	//RE=get_nlines_pesel&NLINES=<liczba linii>&
	//jesli brak zatrejestrowanego PESEL to NLINES=0&
int get_nlines_pesel(const char*in, char*out) {

	int ret;
	char pesel[15];

	select_value(in, "PESEL", pesel);
	if(strcmp(pesel, "NULL") == 0) {
		strcpy(out, "RE=get_nlines_pesel&NLINES=0&");
		return -1;
	}

	//parsowanie "in"
	if(strlen(pesel) != 11) {
		strcpy(out, "RE=get_nlines_pesel&NLINES=0&");
		return -1;
	}

	ret = get_nlines_obj(pesel);
	if(ret == 0) {
		strcpy(out, "RE=get_nlines_pesel&NLINES=0&");
		return -1;
	}

	sprintf(pesel, "%d", ret);

	strcpy(out, "RE=get_nlines_pesel&NLINES=");
	strcat(out, pesel);
	strcat(out, "&");

	return 0;
}



	//pobiera line o podanym numerze (dla OBJ)
	//OP=get_line_pesel_obj&PESEL=...&NLINE=<numer linii>&
	//Odpowiedz:
	//RE=get_line_pesel_obj&STAT=done&PESEL=...&UID32=...&DEP=..&
	//lub
	//RE=get_line_pesel_obj&STAT=done&&STATUS=refused&CP=<powod bledu>&
int get_line_pesel_obj(const char*in, char*out) {

	int ret;
	int nline;
	char pesel[15];
	char snline[10];

	//parsowanie "in'
	ret = 1;
	select_value(in,  "PESEL", pesel);
	select_value(in,  "NLINE", snline);
	if(strcmp(pesel,  "NULL") == 0)  ret = 0;
	if(strcmp(snline, "NULL") == 0)  ret = 0;
	if(strlen(pesel) != 11) ret = 0;
	if(!ret) {
		strcpy(out, "RE=get_line_pesel_obj&STATUS=refused&CP=incorrect_qvector&");
		return -1;
	}
	nline = atoi(snline);
	if(nline < 1) {
		strcpy(out, "RE=get_line_pesel_obj&STATUS=refused&CP=wrong_nline[1]&");
		return -1;
	}

	//pobiera okreslona linie z pliku struktury PESEL
	ret = read_obj_pesel_line(pesel, nline, out);

	if(ret < 0) {
		strcpy(out, "RE=get_line_pesel_obj&STATUS=refused&CP=wrong_nline[2]&");
		return -1;
	}
	
	return 0;

}



	//pobiera node data na ktorym jest zarejestrowane jest konto
	//poprzez UID32/DEP
	//OP=get_node&UID32=...&DEP=..&
	//Zwraca:
	//RE=get_node&STATUS=done&DEP=..&NAME=..&DEP=...&NIP=..
	//&REGON=...&LIP=...&PUID=...&PORTP=..&PORTQ=..&
	//lub jesli byl blad
	//RE=get_node&STATUS=refused&CP=<powod odmowy>&
int get_node(const char*in, char*out) {

	int ret;
	char uid32[35];
	char dep[15];
	char qstr[512];

	ret = 1;
	select_value(in, "UID32", uid32);
	select_value(in, "DEP", dep);
	if(strcmp(uid32, "NULL") == 0)   ret = 0;
	if(strcmp(dep,   "NULL") == 0)   ret = 0;
	if(!ret) {
		strcpy(out, "RE=get_node&STATUS=refused&CP=incorrect_qvector&");
		return -1;
	}

	//pobiera dane z UID32BASE
	ret = get_node_data(uid32, dep, qstr);

	if(ret < 0) {
		if(ret == -2) {
			strcpy(out, "RE=get_node&STATUS=refused&CP=no_ip_node&");
			return -1;
		}
		strcpy(out, "RE=get_node&STATUS=refused&CP=no_data_node&");
		return -1;
	}

	strcpy(out, "RE=get_node&STATUS=done&");
	strcat(out, qstr);

	return 0;
}
////////////////////////////////////////////////////////////////////////////////
//                             END GET DATA FROM MAP
////////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                        (static/var) IP ADDRESS
///////////////////////////////////////////////////////////////////////////////
	//--- dodaje i zmienia (static/var) ip adres UIDL node ---
	//OP=add_ip_node&UID32=..&IPNODE=..&
	//RE=add_ip_node&STATUS=done&
	//lub
	//RE=add_ip_node&STATUS=refused&
int add_ip_node(const char*in, char*out) {

	int ret;
	char uid32[35];
	char ip[20];

	ret = 1;
	select_value(in, "UID32", uid32);
	select_value(in, "IPNODE", ip);
	if(strcmp(uid32, "NULL") == 0) ret = 0;
	if(strcmp(ip, "NULL")    == 0) ret = 0;
	if( !check_ip_form(ip) ) ret = 0; 
	if(!ret) {
		strcpy(out, "RE=add_ip_node&STATUS=refused&CP=incorrect_qvector&");
		return -1;
	}

	//zapisuje do structury UID32BSE
	ret = save_ip_addr(uid32, ip);
	if(ret < 0) {
		strcpy(out, "RE=add_ip_node&STATUS=refused&CP=no_file&");
		return -1;
	}
	
	strcpy(out, "RE=add_ip_node&STATUS=done&");

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                      END (static/var) IP ADDRESS
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                              VERSION UIDC
///////////////////////////////////////////////////////////////////////////////
	//OP=versionUIDC&
	//RE=versionUIDC&VER=..&SYS=...&
	//lub
	//RE=versionUIDC&VER=null&SYS=null&CP=<powod bledu>&
int versionUIDC(const char*in, char*out) {

	char st[128];
	char *p;

	FILE*fd = fopen("BIN/version", "r");
	if(fd == NULL) {
		strcpy(out, "RE=versionUIDC&VER=null&SYS=null&CP=no_file&");
		return -1;
	}

	fgets(st, 128, fd);
		p = strchr(st, '\n');
		if(p != NULL) *p = '\0';
		strcpy(out, "RE=versionUIDC&VER=");
		strcat(out, st);
		strcat(out, "&SYS=");
		fgets(st, 128, fd);
	fclose(fd);

		p = strchr(st, '\n');
		if(p != NULL) *p = '\0';
		strcat(out, st);
		strcat(out, "&");
	return 0; 
}
///////////////////////////////////////////////////////////////////////////////
//                            END VERSION UIDC
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                            RESTORE BACKUP
///////////////////////////////////////////////////////////////////////////////
// --- robi restore plikow z katalogu ../LIST/... do innych katalogow ---
//OP=restore_files_local&
//RE=restore_files_localT&STATUS=<done/refused>CP=<powod odmowy>&
//
//pliki backup:
//		LIST_PESEL_USR "list_pesel_usr.q" //konta user
//		LIST_PESEL_OBJ "list_pesel_obj.q" //konta obiektow
//		LIST_PESEL_NOD "list_pesel_nod.q" //konta nip/regon
//		LIST_UID32BASE "list_uid32base.q" //baza users
//		LIST_START "list_start_base.q" 	//baza kodow start 
//		LAST_NUMERATOR "numerator"
int restore_files_local(const char*in, char*out, const char*ip_client) {

	int ret;
	int qtyp;
	int i;
   long fsize;
   long nlines;	
	char str[512];
	char st[512];
	char abc[15];
	char pesel[15];
	char uid[35];
	char ip[20];
	char path[128];
	char *p;
	FILE*fp;
	
	strcpy(out, "null"); //przydatne przy testach

	//Sprawdza czy odwolanie jest z LAN
	if(!filtr_lan_ip(ip_client, "null")) {
		strcpy(out, "RE=restore_files_local&STATUS=refused&CP=wrong_lan_ip&");
		return -1;
	}


	//<---- 1. restore LAST_NUMERATOR 
	//(cp DISK/LIST/rs_last_number.txt DISK/rs_last_number.txt)
	strcpy(st, "cp ");
	strcat(st, list_base);
	strcat(st, "/");
	strcat(st, LAST_NUMERATOR);
	strcat(st, " ");
	strcpy(str, list_base);
	p = strchr(str, '/');
	if(p != NULL) *p = '\0';
	strcat(st, str);
	strcat(st, "/");
	strcat(st, LAST_NUMERATOR);
	system(st); //kopiowanie plikow


	//<---- czyszczenie katalogu PESEL
   clear_base_PESEL();


	//<--- 2. kopiuje list_pesel_usr.q do struktury PESEL FILES 
	//tworzy sciezke do backup PESEL
	strcpy(str, list_base);
	strcat(str, "/");
	strcat(str, LIST_PESEL_USR);
	fp = fopen(str, "r");
	if(fp == NULL) {
		strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[1]&");
		return -1;

	}
	while(fgets(str, 512, fp) != NULL) {
		select_value(str, "PESEL", pesel);
		if(strcmp(pesel, "NULL") == 0) {
			fclose(fp);
			strcpy(out, 
				"RE=restore_files_local&STATUS=refused&CP=incorrect_qvector[1]&");
			return -1;			
		}
		p = strchr(str, '\n');
		if(p != NULL) *p = '\0';
		//tworzy sciezke pesel (makedir);
		make_dir_pesel(pesel);
   	//tworzy sciezke dostepu do pesel
   	make_path_pesel(pesel, st);
		//strcat(st, "/");
		strcat(st, PESEL_USR);
		//zapis do pliku pesel usr
		ret = add_string_to_file(st, str); 
		if(ret < 0) {
			fclose(fp);
			strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[2]&");
			return -1;
		}
	}
	fclose(fp); //<----- zakonczenie kopiowania do struktury PESEL FILES


	//<--- 2a. kopiuje list_pesel_obj.q do struktury PESEL FILES 
	//tworzy sciezke do backup PESEL
	strcpy(str, list_base);
	strcat(str, "/");
	strcat(str, LIST_PESEL_OBJ);
	fp = fopen(str, "r");
	if(fp == NULL) {
		strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[3]&");
		return -1;

	}
	while(fgets(str, 512, fp) != NULL) {
		select_value(str, "PESEL", pesel);
		if(strcmp(pesel, "NULL") == 0) {
			fclose(fp);
			strcpy(out, 
				"RE=restore_files_local&STATUS=refused&CP=incorrect_qvector[2]&");
			return -1;			
		}
		p = strchr(str, '\n');
		if(p != NULL) *p = '\0';
		//tworzy sciezke pesel (makedir);
		 make_dir_pesel(pesel);
   	//tworzy sciezke dostepu do pesel
   	make_path_pesel(pesel, st);
		strcat(st, "/");
		strcat(st, PESEL_OBJ);
		//zapis do pliku pesel.obj
		ret = add_string_to_file(st, str); 
		if(ret < 0) {
			fclose(fp);
			strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[4]&");
			return -1;
		}
	}
	fclose(fp); //<----- zakonczenie kopiowania do struktury PESEL FILES

	//<--- 2b. kopiuje list_pesel_nod.q do struktury PESEL FILES 
	//tworzy sciezke do backup PESEL
	strcpy(str, list_base);
	strcat(str, "/");
	strcat(str, LIST_PESEL_NOD);
	fp = fopen(str, "r");
	if(fp == NULL) {
		strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[5]&");
		return -1;

	}
	while(fgets(str, 512, fp) != NULL) {

		select_value(str, "NIP", pesel);
		if(strcmp(pesel, "NULL") != 0 && strcmp(pesel, "null") != 0) {
			p = strchr(str, '\n');
			if(p != NULL) *p = '\0';
			//tworzy sciezke pesel (makedir);
		 	make_dir_pesel(pesel);
   		//tworzy sciezke dostepu do pesel
   		make_path_pesel(pesel, st);
			strcat(st, "/");
			strcat(st, PESEL_NOD);
			//zapis do pliku nod.q
			ret = add_string_to_file(st, str); 
			if(ret < 0) {
				fclose(fp);
				strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[6]&");
				return -1;
			}
		}
		select_value(str, "REGON", pesel);
		if(strcmp(pesel, "NULL") != 0 && strcmp(pesel, "null") != 0) {
			p = strchr(str, '\n');
			if(p != NULL) *p = '\0';
			//tworzy sciezke pesel (makedir);
		 	make_dir_pesel(pesel);
   		//tworzy sciezke dostepu do pesel
   		make_path_pesel(pesel, st);
			strcat(st, "/");
			strcat(st, PESEL_NOD);
			//zapis do pliku nod.q
			ret = add_string_to_file(st, str); 
			if(ret < 0) {
				fclose(fp);
				strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[6]&");
				return -1;
			}
		}

	}
	fclose(fp); //<----- zakonczenie kopiowania do struktury PESEL FILES


	//<---- czyszczenie struktury UID32BASE  
	clear_UID32BASE();


	//<---- 3. Kopiowanie danych z list_uid32base.q do struktury UID32BASE
	//(odbudowuje node.q oraz usr.q) 
	//node.q - dane node
	//usr.q  - dane uzytkownika
	//odczytuje z pliku backup 
	//zapis backup
	strcpy(str, list_base);
	strcat(str, "/");
	strcat(str, LIST_UID32BASE);

	fp = fopen(str, "r");
	if(fp == NULL) {
		strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[7]&");
		return -1;

	}
	while(fgets(str, 512, fp) != NULL) {
		select_value(str, "UIDP", uid);
		qtyp = 1;
		if(strcmp(uid, "NULL") == 0) {
			select_value(str, "UID32", uid);
			qtyp = 0;
			if(strcmp(uid, "NULL") == 0) {
				fclose(fp);
				strcpy(out, 
				"RE=restore_files_local&STATUS=refused&CP=incorrect_qvector[4]&");
				return -1;	
			}	
		}
		p = strchr(str, '\n');
		if(p != NULL) *p = '\0';
		//siezka dostepu do q-vector user
		//fragment path uid /X/X/X/ 
		st_abc(uid, abc);
		strcpy(st, uid32base);
		strcat(st, abc);
		strcat(st, uid);
		//tworzy katalog o nazwie user_qvector.uidp
		mkdir(st, 0777 & ~022); // drugi kawalek
		strcat(st, "/");
		if (qtyp) strcat(st, QUSR); //plik danych usera
			else strcat(st, QNODE);  //plik danych node
		//zapis do UID32BASE
		ret = add_string_to_file(st, str);
		if(ret < 0) {
			fclose(fp);
			strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[8]&");
			return -1;
		}
	}
	fclose(fp); //<--- zakonczenie do usr.q oraz node.q w strukturze UID32BASE




	//<--- 3a. Kopiowanie danych z list_ip.nq do struktury UID32BASE (ip.q)
	//odczytuje z pliku backup (UID32=..&IPADDR=..&)
	//zapis backup
	strcpy(str, list_base);
	strcat(str, "/");
	strcat(str, LAST_IPNODE);

	//Uwaga, plik  LAST_IPNODE (list_ip.nq) jest zapisany w trybie binarnym
	//dlatego liczbe linii nalezy obliczyc.
	//Dlugosc linii w pliku list_ip.nq jest stala i wynosi 65 chars
	fp = fopen(str, "r");

	if(fp == NULL) {
		strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[9]&");
		return -1;
	}

	//oblicza liczbe linii w pliku ip.q (LAST_IPNODE)
   fsize = fseek(fp, 0L, SEEK_END);	//koniec pliku
   fsize = ftell(fp);					// size of file
   fseek(fp, 0L, SEEK_SET); 			// to samo co rewind(fp)
   nlines = fsize/65;					// oblicza liczbe linii (o len 65 chars)

	/*
 	//odczyt w trybie textowym
	while(fgets(str, 512, fp) != NULL) {
	}*/

	//odczyt linii w trybie binarnym 
	//(w pliku  LAST_IPNODE (list_ip.nq) linie zapisane sa w trybie binarnym)
   for(i=0; i < (int)nlines; i++) {

		fread(str, 65, 1, fp); //odczytuje linie o dlugosci 65 bytes

		//wyluskuje wartosci q-versorow
		select_value(str, "UID32", uid);
		if(strcmp(uid, "NULL") == 0) {
				fclose(fp);
				strcpy(out, 
				"RE=restore_files_local&STATUS=refused&CP=incorrect_qvector[5]&");
				return -1;	
		}	
		select_value(str, "IPADDR", ip);
		if(strcmp(ip, "NULL") == 0) {
				fclose(fp);
				strcpy(out, 
				"RE=restore_files_local&STATUS=refused&CP=incorrect_qvector[6]&");
				return -1;	
		}

		//przepisuje qstring po parsowaniu
		strcpy(str, "IPADDR=");
		strcat(str, ip);
		strcat(str, "&");	
		
		p = strchr(str, '\n');
		if(p != NULL) *p = '\0';
		//siezka dostepu do q-vector user
		//fragment path uid /X/X/X/ 
		st_abc(uid, abc);
		strcpy(st, uid32base);
		strcat(st, abc);
		strcat(st, uid);
		//tworzy katalog o nazwie user_qvector.uidp
		mkdir(st, 0777 & ~022); // drugi kawalek
		strcat(st, "/");
		strcat(st, IPNODE); //plik danymi ip
		//zapis do UID32BASE (ip.q)
		ret = add_string_to_file(st, str);
		if(ret < 0) {
			fclose(fp);
			strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[10]&");
			return -1;
		}
	}
	fclose(fp);//<------ zakonczenie kopiowania do ip.q w strukturze UID32BASE



	//<----- czyszczenie i odbudowanie struktury START
	clear_base_START();

	//<----- 4.Kopiowanie danych z LIST_START (list_start_base.q)
	//plik backup
	//postac linii: CODE=...&VDATE=..& 
   strcpy(str, list_base);
	strcat(str, "/");
	strcat(str, LIST_START);

	//otwiera plik backup
	fp = fopen(str, "r");

	if(fp == NULL) {
		strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[11]&");
		return -1;
	}

	while(fgets(str, 512, fp) != NULL) {

		p = strchr(str, '\n');
		if(p != NULL) *p = '\0';

		select_value(str, "CODE", st);
		if(strcmp(st, "NULL") == 0) {
				fclose(fp);
				strcpy(out, 
				"RE=restore_files_local&STATUS=refused&CP=incorrect_qvector[7]&");
				return -1;	
		}

		// Zapisuje do START (bazy)
		st_abc(st, abc);
		strcpy(path, start_base); 
		strcat(path, abc);
		if(add_text_line(path, SCODE, str) < 0) {
			strcpy(out, "RE=restore_files_local&STATUS=refused&CP=no_file[12]&");
			return -1;
		}

	}
	fclose(fp);//<------ zakonczenie kopiowania do scode.q w strukturze START


	strcpy(out, "RE=restore_files_local&STATUS=done&");
		

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                          END RESTORE BACKUP
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                  ZMIANA USTAWIEN IP/PORT UIDC W CONFIGU
///////////////////////////////////////////////////////////////////////////////
	//pobiera IP/PORT UIDC z configu UIDC
	//OP=get_uidc_config&
	//RE=get_uidc_config&PORTUIDC=...&
	//Jesli blad to zwraca:
	//RE=get_uidc_config&PORTUIDC=null&
int get_uidc_config(const char*in, char*out) {

	char*fname = CONFIG_FILENAME;
	char st[126]; 
	char port_uidc[10] = "null";
	char*p;	
	
	FILE*fd = fopen(fname, "r");
	if(fd == NULL) {
		strcpy(out, 
		  "RE=get_uidc_config&IPUIDC=null&PORTUIDC=null&CP=no_file&");
		return -1;
	}
	//odczytuje ustawienia z pliku mserverUIDL.config
	while(fgets(st, 126, fd) != NULL) {
		p = strstr(st, "PORT_CENTRAL");
		if(p != NULL) {
			select_value(st, "PORT_CENTRAL", port_uidc); 
		}
	}

	fclose(fd);

	if(strcmp(port_uidc, "NULL") == 0) strcpy(port_uidc, "null");

	strcpy(out,  "RE=get_uidc_config&PORTUIDC=");
	strcat(out, port_uidc);
	strcat(out, "&");

	return 0;
}



	//zmienia IP/PORT UIDC w pliku konfiguracyjnym UIDC
	//OP=chg_uidc_config&PORTUIDC=...&
	//RE=chg_uidc_config&STATUS=<done/refused>CP=<powod odmowy>&
	//lun w przypadku bledu
	//RE=chg_uidc_config&STATUS=<done/refused>CP=<powod odmowy>&
int chg_uidc_config(const char*in, char*out, const char*ip_client) {

	char*fname = CONFIG_FILENAME;
	char*fntmp = "mserverUIDL.tmp";
	char st[126]; 
	char port_uidc[8] = "null";
	char*p;
	
	//Sprawdza czy odwolanie jest z LAN
	if(!filtr_lan_ip(ip_client, "null")) {
		strcpy(out, "RE=chg_uidc_config&STATUS=refused&CP=wrong_lan_ip&");
		return -1;
	}

	//rozklada qstring "in" 
	select_value(in, "PORTUIDC", port_uidc);
	if(strcmp(port_uidc, "NULL") == 0) {
		strcpy(out, 
         "RE=chg_uidc_config&STATUS=refused&CP=incorrect_qvector&");
		return -1;
	} 

	FILE*fd = fopen(fname, "r");
	if(fd == NULL) {
		strcpy(out, 
		  "RE=get_uidc_config&IPUIDC=null&PORTUIDC=null&CP=no_file[1]&");
		return -1;
	}

	FILE*fw = fopen(fntmp, "w");
	if(fd == NULL) {
		fclose(fd);
		strcpy(out, 
		  "RE=get_uidc_config&IPUIDC=null&PORTUIDC=null&CP=no_file[2]&");
		return -1;
	}
	while(fgets(st, 126, fd) != NULL) {
		p = strstr(st, "PORT_CENTRAL");
		if(p != NULL) {
			strcpy(st, "PORT_CENTRAL=");
			strcat(st, port_uidc); 
			strcat(st, "&");
			fprintf(fw, "%s\n", st);
		} else {
				p = strchr(st, '\n');
				if(p != NULL) *p = '\0';
				fprintf(fw, "%s\n", st);
			}
		
	}
	fclose(fd);
	fclose(fw);

	remove(fname);
	rename(fntmp, fname);

	strcpy(out, "RE=chg_uidc_config&STATUS=done&");

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                END ZMIANA USTAWIEN IP/PORT UIDC W CONFIGU
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
//                              LOG & LOG FILE
///////////////////////////////////////////////////////////////////////////////
	//zapisuje ostatnie logowanie admina
	//OP=log_adm&PESEL=...&NICK=..&
	//RE=log_adm&STATUS=done&
	//lub
	//RE=log_admin&STATUS=refused&CP=<powod odmowy>&
int log_adm(const char*in, char*out) {

	int ret;
	char pesel[80];
	char nick[80];
	
	//parsing string "in"
	ret = 1;
	select_value(in, "PESEL", pesel);
	select_value(in, "NICK", nick);
	if(strcmp(pesel, "NULL") == 0) ret = 0;
	if(strcmp(nick,  "NULL") == 0) ret = 0;
	if(!ret) {
	strcpy(out, "RE=log_adm&STAUS=refused&CP=incorrect_qvector[1]&");
		return -1;
	}

	if(strlen(nick) > 20) {
		strcpy(out, "RE=log_adm&STAUS=refused&CP=incorrect_qvector[2]&");
		return -1;
	}
	if(strlen(pesel) != 11) {
		strcpy(out, "RE=log_adm&STAUS=refused&CP=incorrect_qvector[3]&");
		return -1;
	}

	//sprawdza formule pesel
	ret = check_form_pesel(pesel);
	if(!ret) {
	strcpy(out, "RE=log_adm&STAUS=refused&CP=incorrect_qvector[4]&");
		return -1;
	}

	//loguje
	ret = log_file(pesel, nick);
	if(ret < 0) {
	strcpy(out, "RE=log_adm&STAUS=refused&CP=no_log&");
		return -1;
	}

	strcpy(out, "RE=log_adm&STAUS=done&");

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
//                             LOG & END LOG FILE
///////////////////////////////////////////////////////////////////////////////

