/******************************************************************************
 *
 *
 *				CODER/DECODER
 *
 *  - USER PIN/PASSWD
 *  - UID/SHORT MESSAGES
 *
 *
 *****************************************************************************/



#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <time.h>
#include <stdint.h>
#include <sys/ioctl.h>
#include <linux/fs.h>
#include <sys/types.h>
#include <sys/stat.h>
//#include <math.h>
#include "genRSA.c"


//definiuje ANSI kolory w printf
#define BOLDBLACK   "\033[1m\033[30m"
#define BOLDRED     "\033[1m\033[31m"
#define BOLDBLUE    "\033[1m\033[34m"
#define RED     	"\x1b[31m"
#define GREEN   	"\x1b[32m"
#define YELLOW  	"\x1b[33m"
#define BLUE    	"\x1b[34m"
#define MAGENTA 	"\x1b[35m"
#define CYAN    	"\x1b[36m"
#define COLOR_RESET   	"\x1b[0m"



//defined in confQ.c
//#define HEADSIZE 512  //cuder V1(for uid)
//#define BUFSIZE HEADSIZE+512

unsigned int key=63542357;//value isn't important





//------------------------ KEY_PASS & NOISE GENERATORS ------------------------ 
//returns:
//       1 if /dev/urandom works correctly
//       0 if noise was generated by software random
// data - buffer for noise
// bytes_count i-  number of noise samples       
int noise_gen(char*data, int byte_count) {

   //in WIN rand_s()
   struct timespec tms;
   int i;
   char bc[64];
   FILE*fp;

  
   fp = fopen("/dev/urandom", "r");
   if(fp==NULL) goto label_end;
   for(i=0; i<byte_count; i++) {
      fread(&bc, 1, 64, fp);
      if(bc[17]!=0) *(data+i) = bc[17];
         else if(bc[23]!=0) *(data+i) = bc[23];
            else if(bc[43]!=0) *(data+i) = bc[43];
   }
   fclose(fp);
   return 1;

//if urandom doesn't work corectly
label_end:
   //accuracy ns
   if(clock_gettime(CLOCK_REALTIME, &tms) == -1 ) {
      //static noise if real clock doesn't work 
      //(very small likelihood for this place) 
      srand(24749183);
      for(i=0; i<286;i++) rand();
      for(i=0; i<byte_count; i++) data[i]=rand()%255;
      return 0;
    }

    unsigned int seed = 
     ((unsigned int)(tms.tv_sec + (time_t)tms.tv_nsec))%4294967293;
    
    srand(seed);
    for(i=0; i<257; i++) rand();
    for(i=0; i<byte_count; i++) data[i]=rand()%255;
    return 0;
}


//key for coder/decoder 
//  seed for RSA_srand changes automatically after 24 hours
//  returns:
//         -1 if time() isn't available and see=19700211
//          0 - correct state
int key_number(unsigned int*seed) {
    
    struct tm *tms;
    time_t tt = time(NULL);
    int y,m,d;

    if(tt==(time_t)-1) {*seed=19700211; return -1;}

    tms = localtime(&tt);

    y = tms->tm_year+1900;
    m = tms->tm_mon;
    d = tms->tm_yday+1;   

    *seed = (unsigned int)(17*(d*123+y*11+m));

    return 0; 
}
//--------------------- END KEY_PASS & NOISE GENERATORS -----------------------




//------------------ PRIVATE USER's PIN/PASSWD CODER/DECODER ------------------

////char uidp[32]  = "EXZ1234567890123456789012345678";
///char euidp[32] = "WXZ84994750p3478[09u34thi8yh04f";
//0-4294967295
//256power(4) = 4294967296 
// buf[4096];
// creation privete user's passwd for MMC/USB
void usb_passwd_gen(const char*user_key, char*buf) {

   char key[256];
   char cover[256];
   int i,j,len,shift;
   char k[2];   

   if(user_key==NULL) {buf[0]='\0'; return;}
   len = (int)strlen(user_key);
   if(len==0)  {buf[0]='\0'; return;}

   if(len>255) {
      memcpy(key,user_key,254);
      key[255]='\0';
   } else strcpy(key,user_key);

   len = (int)strlen(key);

   shift = (4096/len) - 7;
   
   //encapsulation
   RSA_srand(len);
   for(i=0; i<256; i++) cover[i]=RSA_rand()%255;

   //noise
   RSA_srand(time(NULL));
   for(i=0; i<4096; i++) buf[i]=RSA_rand()%255;

   j=0;
   for(i=0; i<len; i++) {
      RSA_srand((unsigned int)key[i]);
      k[0] = (char)(RSA_rand()%255);
      //j+=12;
      j+=shift+abs(k[0]%7);
      buf[j]=k[0]^cover[i];
   } 
}


//check key stored in dongle
//private user's passsd of MMC/USB
//returns:
//      1 - user_key is correct
//      0 - User_key is incorrect
int decode_usb_passwd(const char*user_key, const char*buf) {
   char key[256];
   char cover[256];
   int i,j,len,shift;
   char k[2];
  
   if(user_key==NULL) return 0;
   len = (int)strlen(user_key);
   if(len==0) return 0;
   
   if(len>255) {
      memcpy(key,user_key,254);
      key[255]='\0';
   } else strcpy(key,user_key);

   len = (int)strlen(key);

   shift = (4096/len) - 7;

   RSA_srand(len);
   for(i=0; i<256; i++) cover[i]=RSA_rand()%255;

   j=0;
   for(i=0; i<len; i++) {
      RSA_srand((unsigned int)key[i]);
      k[0] = (char)(RSA_rand()%255);
       j+=shift+abs(k[0]%7);
      if(buf[j]!=(k[0]^cover[i])) return 0;
   }
   return 1;
}
//-------------- END PRIVATE USER's PIN/PASSWD CODER/DECODER ------------------




//---------------------- UID/SHORT MESSAGES CODER -----------------------------
// out size = 2*512;
// limit text size < 512;
void codeV1(const char*text, unsigned int key, char*out) {

   int i,j; 
   
   char key_buf1[HEADSIZE]; //512 bytes
   char key_buf2[HEADSIZE];
   char text1[HEADSIZE];

   size_t len = strlen(text);
   if(len>HEADSIZE) len=HEADSIZE-1;      //cuts length message to 512 bytes  

   noise_gen(text1, HEADSIZE);
   memcpy(text1, text, len+1);
   
   key_number(&key); //gets current passwd

   noise_gen(key_buf1, HEADSIZE);

   RSA_srand(key);
   for(i=0; i<HEADSIZE; i++) key_buf2[i]=RSA_rand()%255;

   i=0;j=0;
   while(i<HEADSIZE) {
     out[j]=text1[i]^key_buf1[i];
     out[j+1]=key_buf1[i]^key_buf2[i];
    i++;
    j+=2;
   }
}


//out size = 2*512;
//limt text size < 512;
void decodeV1(char*out, unsigned int key, char*text) {

   int i,j;
   char key_buf2[HEADSIZE];
   
    key_number(&key);

   RSA_srand(key);
   for(i=0; i<HEADSIZE; i++) key_buf2[i]=RSA_rand()%255;

   i=0;j=0;
   while(i<HEADSIZE) {
     out[j+1]=out[j+1]^key_buf2[i];
     text[i]=out[j]^out[j+1];
    i++;
    j+=2;
   }
}
//---------------------- END UID/SHORT MESSAGES CODER -------------------------






/*
int main() {

char txt[HEADSIZE] = "to jest test";
char out[BUFSIZE];
int i,j;

//test noise gen
for(j=0; j<10; j++) {
   i = noise_gen(txt, HEADSIZE);
   printf("i=%d\n", i);
   //for(i=0; i<10; i++)  printf("%d ",abs(txt[i]));
   //printf("\n");
   for(i=0; i<10; i++)  printf("%d ",txt[i]);
   printf("\n");
}
return 0;


for(j=0; j<10; j++) {
   printf("[0] txt=%s\n", txt);
   codeV1(txt, 0, out);

   //for(i=0; i<20; i++)  printf("%d ",abs(out[i]));
   for(i=0; i<20; i++)  printf("%c ",out[i]);
   txt[0]='\0';
   printf("\n");

   decodeV1(out,0, txt);
   printf("[1] txt=%s\n", txt);
}

return 0;
}*/
