///////////////////////////////////////////////////////////////////////////////
//
//                    Wczytuje dane z pliku konfiguracyjnego
//
//                              (JPW-2019)
//
//
///////////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#ifndef SELECT 
#define SELECT
#include "SRC/select_value.c" 
#endif



#define CONFIG_FILENAME "mserverUIDC.config"


// --- zmienne globalne ---

#define PORT_CENTRAL 8809 //definjujemy port nasluchiwania
int  port_central = PORT_CENTRAL;

//polozenie bazy UID dla generowanych UID32
#define BASE_UID "DISK/UID32BASE" //path do bazy UID32
char uid32base[512]=BASE_UID;

#define BASE_PESEL "DISK/PESEL" //path do katalogu PESEL
char pesel_base[512] = BASE_PESEL;

#define LIST_BASE "DISK/LIST"
char list_base[256] = LIST_BASE;

#define START_BASE "DISK/START"
char start_base[256] = START_BASE;



#define TEST_MODE 0 //0-nie robi printf, 1 -pokazuje na ekranie
int test_mode=TEST_MODE;

// --- koniec zmienne globalne ---




// --- zmienne dla montowania dyskow ---

#define MOUNTPOINT "null"
char mount_point_UID[20] = MOUNTPOINT;
char mount_point_PESEL[20] = MOUNTPOINT;
char mount_point_LIST[20] = MOUNTPOINT;
char mount_point_START[20] = MOUNTPOINT;

// --- koniec zmiennych dla montowania dyskow ---





//odczytuje dane z pliku konfiguracyjnego
int read_config_file(void) {

	char*p;

	char*st  = (char*)malloc(1000*sizeof(char));
	char*out = (char*)malloc(1000*sizeof(char));

	FILE*fd = fopen(CONFIG_FILENAME,"r");
	if(fd==NULL) {free(st); free(out); return -1;}

	while(fgets(st,1000,fd)!=NULL) {

//printf("st: %s\n", st);
				
		p = strstr(st, "PORT_CENTRAL");
		if(p != NULL && st[0] != '#') {
	   		select_value(st,"PORT_CENTRAL",out);
	   		if(strcmp(out,"NULL")!=0) port_central = atoi(out);
		}

		//polozene bazy UID32BASE
		p = strstr(st, "UIDBASE");
		if (p != NULL && st[0] != '#') {
			select_value(st,"UIDBASE",out);
			if(strcmp(out,"NULL")!=0) strcpy(mount_point_UID, out);
			select_value(st, "DIR", out);
			if(strcmp(out, "NULL") != 0) {
			    strcpy(uid32base, "DISK");
			    strcat(uid32base, "/");
			    strcat(uid32base, out);

			}
		}

		//polozenie katalogu PESEL
		p = strstr(st, "PESELBASE");
		if (p != NULL && st[0] != '#') {
			select_value(st,"PESELBASE",out);
			if(strcmp(out,"NULL")!=0) strcpy(mount_point_PESEL, out);
			select_value(st, "DIR", out);
			if(strcmp(out, "NULL") != 0) {
			    strcpy(pesel_base, "DISK");
			    strcat(pesel_base, "/");
			    strcat(pesel_base, out);
			}
		}

		//polozenie katalogu LIST
		p = strstr(st, "LISTBASE");
		if (p != NULL && st[0] != '#') {
			select_value(st,"LISTBASE",out);
			if(strcmp(out,"NULL")!=0) strcpy(mount_point_LIST, out);
			select_value(st, "DIR", out);
			if(strcmp(out, "NULL") != 0) {
			    strcpy(list_base, "DISK");
			    strcat(list_base, "/");
			    strcat(list_base, out);
			}

		}

		//polozenie katalogu START
		p = strstr(st, "STARTBASE");
		if (p != NULL && st[0] != '#') {
			select_value(st,"STARTBASE",out);
			if(strcmp(out,"NULL")!=0) strcpy(mount_point_START, out);
			select_value(st, "DIR", out);
			if(strcmp(out, "NULL") != 0) {
			    strcpy(start_base, "DISK");
			    strcat(start_base, "/");
			    strcat(start_base, out);
			}

		}

		//flaga "show internal errors" przy testowaniu serwera
		p = strstr(st, "TEST");
		if (p != NULL && st[0] != '#') {
			select_value(st,"TEST",out);
			if(strcmp(out,"1")==0) test_mode=1;
			else test_mode=0;
		}
	}
	


	free(st);
	free(out);
	fclose(fd);
	return 0;
}


//sprawdza czy mount points sa takie same 
//(UWAGA! montuje katalogi tylko na jednej partycji)
int check_mount_points(char*mount_point) {

	if(strcmp(mount_point_UID, "null") == 0 ||
	   strcmp(mount_point_PESEL, "null") == 0 ||
	   strcmp(mount_point_LIST, "null") == 0 ||
		strcmp(mount_point_START, "null") == 0) {
		strcpy(mount_point, "null");
		return 1;
	}


	if(strcmp(mount_point_UID, mount_point_PESEL) != 0) return 0;
	if(strcmp(mount_point_UID, mount_point_LIST) != 0) return 0;
	if(strcmp(mount_point_UID, mount_point_PESEL) != 0) return 0;
	if(strcmp(mount_point_PESEL, mount_point_LIST) != 0) return 0;
	if(strcmp(mount_point_PESEL, mount_point_START) != 0) return 0;

	strcpy(mount_point, mount_point_UID);
	return 1;
}


//odmontowuje dysk
int umount_disk(const char*disk) {
 	char st[256] = "umount ";
	strcat(st, disk);

	FILE*fp = popen(st, "r");
	if(fp == NULL) return -1;
	pclose(fp);
	return 0;
}


//montuje dyski
int mmount_disk(const char*disk, const char*dir) {

	char st[256];
        FILE* fp;

	strcpy(st, "mount ");
	strcat(st, disk); 
	strcat(st, " ");
	strcat(st, "DISK");
	fp = popen(st, "r");
	if(fp == NULL) return -1;
	while(fgets(st, 256, fp) != NULL) printf("%s\n", st);
	pclose(fp);
	return 0;
}


//sprawdza czy partycja istniejs
int check_partition_disk(const char*disk) {

    char*s = "fdisk -l";
    char *p;
    char buf[256];

    FILE*fp = popen(s, "r");
    if(fp == NULL) return -1;

    int ch = 0;
    while(fgets(buf, 256, fp) != NULL) {
	p = strstr(buf, disk);
	if(p != NULL) ch = 1;
    }

    pclose(fp);
    return ch;
}


//sprawdza czy partycja jest zamontowana
int check_mounted_disk(const char*disk) {

    char*s = "df -h";
    char *p;
    char buf[256];

    FILE*fp = popen(s, "r");
    if(fp == NULL) return -1;

    int ch = 0;
    while(fgets(buf, 256, fp) != NULL) {
	p = strstr(buf, disk);
	if(p != NULL) ch = 1;
    }

    pclose(fp);
    return ch;
}



//montuje disk   
int mount_folders(void) {

   char disk[20];
   if( read_config_file() < 0) return -1;
   if( !check_mount_points(disk) ) return -1;
   if( strcmp(disk, "null") !=0 ) {
	//sprawdza obecnosc partycji deklarowanej w configu
	if(!check_partition_disk(disk)) {
    		printf("\n %s in config is not connected!\n", disk);
   		return -1;
	}
	
	//sprawdza czy partycja jest zamontowana
	if(check_mounted_disk(disk)) umount_disk(disk);

	sleep(1);

	//montuje dysk
	if(strcmp(disk, "null") != 0) mmount_disk(disk, "DISK");
   } 
    
   return 0;
}




//pobieranie danych z configu oraz montowanie katalogow
int read_config(void) {

  //sprawdza czy logowanie jest na root
  if(getuid() != 0) {
		printf("\n Login as root!\n\n");
		exit(0);
  }

  if(mount_folders() < 0) {
	printf("\n ERROR: incorrct %s\n", CONFIG_FILENAME);
	return -1;
   }

	printf("\n");

	//punkty montowania katalogow 
   if(strcmp(mount_point_UID, "null") == 0) {
		printf(" MOUNT POINT UID:   on system disk\n");
	} else {
 		printf(" MOUNT POINT UID:   %s\n", mount_point_UID);
	}
   if(strcmp(mount_point_PESEL, "null") == 0) {
		printf(" MOUNT POINT PESEL: on system disk\n");
	}
	else {
		printf(" MOUNT POINT PESEL: %s\n", mount_point_PESEL);
	}
	if(strcmp(mount_point_LIST, "null") == 0) {
		printf(" MOUNT POINT LIST:  on system disk\n");
    }	
	else {
    	printf(" MOUNT POINT LIST:  %s\n", mount_point_LIST);
	}

	if(strcmp(mount_point_START, "null") == 0) {
		printf(" MOUNT POINT START: on system disk\n");
    }	
	else {
    	printf(" MOUNT POINT START: %s\n", mount_point_LIST);
	}

   printf(" CENTRAL PORT:      %d\n", port_central);
   printf(" FOLDER UID:        %s\n", uid32base);
   printf(" FOLDER PESEL:      %s\n", pesel_base);
   printf(" FOLDER LIST:       %s\n", list_base);
   printf(" FOLDER START:      %s\n", start_base);

   printf(" TEST:              %d\n\n", test_mode);

    return 0;
}










/*
int main(void) {


    if(mount_folders() < 0) printf("ERROR: brak folderow!\n");

    printf("MOUNT POINT:       %s\n", mount_point_UID);
    printf("MOUNT POINT:       %s\n", mount_point_PESEL);
    printf("MOUNT POINT:       %s\n", mount_point_LIST);
    printf("MOUNT POINT:       %s\n", mount_point_START);

    printf("CENTRAL PORT:  %d\n", port_central);
    printf("BASE:          %s\n", uid32base);
    printf("PESEL:         %s\n", pesel_base);
    printf("LIST:          %s\n", list_base);
    printf("START:         %s\n", start_base);

    printf("TEST:          %d\n", test_mode);

    return 0;
} */
