///////////////////////////////////////////////////////////////////////////////
//
//                                ARCHI
//	               Rozszerzenie do exec_functions_UIDC.c
//	                             (library)
//	                              ver 0.2
//	                            (JPW - 2019)
//
//////////////////////////////////////////////////////////////////////////////



#include <stdio.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#ifndef SELECT 
#define SELECT
#include "SRC/select_value.c" 
#endif

#ifndef READ_CONFIG 
#define READ_CONFIG
#include "SRC/read_config.c"
#endif


#ifndef CLIENTUID
#define CLIENTUID
#include "SRC/clientUID_exec_func.c"
#endif


// --- sciezka do struktury ---
#define PATH "DISK"
// --- end sciezka do struktury ---



// --- LIST FILES ---
// listy bezpieczenstwa do odtworznie struktury w przypadlu awarii
#define LIST_PESEL_USR "list_pesel_usr.q" 	//konta user
#define LIST_PESEL_OBJ "list_pesel_obj.q" 	//konta obiektow
#define LIST_PESEL_NOD "list_pesel_nod.q" 	//konta nip/regon
#define LIST_UID32BASE "list_uid32base.q" 	//baza users
#define LIST_START "list_start_base.q" 		//baza kodow start 
#define LAST_NUMERATOR "rs_last_number.txt" 	//numerator
#define LAST_IPNODE "list_ip.nq" 				//backup ip (static/var)
// --- END LIST FILES ---


// ---  PESEL FILES ---
#define PESEL_USR "pesel.usr" //dla usera
#define PESEL_OBJ "pesel.obj" //dla obiektu 
#define PESEL_NOD "pesel.nod" //dla NIP,REGON (N.., RN...)
// --- END PESEL FILES ---


// --- PLIKI UID32BASE ---
#define QNODE "node.q" 	//dane node
#define QUSR "usr.q" 	//dane uzytkowika
#define IPNODE "ip.q"	//adress ip (IPADDR=..&) 
// --- END PLIKU UID32BASE ---


// --- PLIKI START BASE ---
#define SCODE "scode.q" //plik zawiera kody rejestracyjne nowego UIDL
// --- PLIKI START BASE ---


#define LOGFILE "log.q" 


//---------------------------- Check client IP --------------------------------
// --- filtr ip_client --
// jesli testowane sa IP w LAN to wan_ip="null"
// jesli testowany jest adres w WAN to wan_ip=IP_UIDC
// (bool reply)
int filtr_lan_ip(const char*ip_client, const char*wan_ip) {

	char ip_tmp[20];
	char*p;

	strcpy(ip_tmp, ip_client);

	p = strchr(ip_tmp, '.');
	if(p != NULL) *p = '\0';
	else return 0;

	//bada zgodnosc z WAN tzn. jesli wan_ip != null
	p = strstr(wan_ip, "null");
	if(p == NULL) {
	   if(strcmp(ip_client, wan_ip) == 0) return 1;
	}

	//bada LAN adres (127.x.x.x, 192.x.x.x, 10.x.x.x)
	if(strcmp(wan_ip, "null") == 0) {
		
		p = strstr(ip_tmp, "127");
		if(p != NULL) return 1;

		p = strstr(ip_tmp, "192");
		if(p != NULL) return 1;

		p = strstr(ip_tmp, "10");
		if(p !=NULL) return 1;
	}

	return 0;
}
//-------------------------- END Check client IP ------------------------------




//----------------------- SPRAWDZANIE FORMUL PESEL ----------------------------
//sprawdza formule pesel/nip/regon
int check_form_pesel(const char*pesel) {

	int sum;
	int i;

//printf("NIP: %s\n", pesel);

	if(strlen(pesel) != 11) return 0; //za krotki lub za dlugi PESEL
	if(strcmp(pesel, "N0000000000") == 0) return 0; //odrzucony NIP

	//selekcja pesel, nip, regon, konto specjalne
	if(pesel[0] == 'N') goto label_nip;
	if(pesel[0] == 'R' && pesel[1] == 'N') goto label_regon;
	if(pesel[0] == 'R' && pesel[1] == 'S') goto label_specjal;

	//sprawdza wytepowanie cyfr w stringu pesel
	for(i=0; i<11; i++) {
	   if( *(pesel+i)<'0' && *(pesel+i)>'9') return 0;
	}
	
	//sprawdza formule pesel
	//9×a + 7×b + 3×c + 1×d + 9×e + 7×f + 3×g + 1×h + 9×i + 7×j
	sum = 	9*(pesel[0] - 48) +
		7*(pesel[1] - 48) +
		3*(pesel[2] - 48) +
		1*(pesel[3] - 48) +
		9*(pesel[4] - 48) +
		7*(pesel[5] - 48) +
		3*(pesel[6] - 48) +
		1*(pesel[7] - 48) +
		9*(pesel[8] - 48) +
		7*(pesel[9] - 48);
	//oblicza reszte
	sum %= 10;

	//sprawdza reszte z ostania cyfra
	if(sum == pesel[10]-48) return 1;
	else return 0;

label_nip:
//puts("---nip");
	//sprawdza cyfry w stringu pesel
	for(i=1; i<11; i++) {
	   if( *(pesel+i)<'0' && *(pesel+i)>'9') return 0;
	}

	//sprawdza formule nip
	//wagi 6, 5, 7, 2, 3, 4, 5, 6, 7
	sum =	6*(pesel[1] - 48) +
		5*(pesel[2] - 48) +
		7*(pesel[3] - 48) +
		2*(pesel[4] - 48) +
		3*(pesel[5] - 48) +
		4*(pesel[6] - 48) +
		5*(pesel[7] - 48) +
		6*(pesel[8] - 48) +
		7*(pesel[9] - 48);
	//oblicza reszte
	sum %= 11;
//printf("SUM: %d\n", sum);
	if(sum == pesel[10]-48) return 1;
	else return 0;
	

label_regon:

	//sprawdza cyfry w stringu pesel
	for(i=2; i<11; i++) {
	   if( *(pesel+i)<'0' && *(pesel+i)>'9') return 0;
	}

	//sprawdza formule regon
	//wagi: 8, 9, 2, 3, 4, 5, 6, 7
	sum = 	8*(pesel[2] - 48) +
		9*(pesel[3] - 48) +
		2*(pesel[4] - 48) +
		3*(pesel[5] - 48) +
		4*(pesel[6] - 48) +
		5*(pesel[7] - 48) +
		6*(pesel[8] - 48) +
		7*(pesel[9] - 48);
	//oblicza reszte
	sum %= 11;
	if(sum == 10) sum = 0;
	if(sum == pesel[10]-48) return 1;
	else return 0;


label_specjal:

	//sprawdza cyfry w stringu specjalnym
	sum = 1;
	for(i=2; i<11; i++) {
	   if( *(pesel+i)<'0' && *(pesel+i)>'9') sum = 0;
	}
	if(sum) return 1;

	return 0;
}
//--------------------- END SPRAWDZANIE FORMUL PESEL --------------------------



//----------------------------- PATHS CREATION --------------------------------
//tworzy sciezke dostepu PESEL
int make_path_pesel(const char*pesel, char *path) {
	if(strlen(pesel)!=11) return -1;

	char str[20];

	str[0]=pesel[0];
	str[1]=pesel[1];
	str[2]='/';
	str[3]=pesel[2];
	str[4]=pesel[3];
	str[5]='/';
	str[6]=pesel[4];
	str[7]=pesel[5];
	str[8]='/';
	str[9]=pesel[6];
	str[10]=pesel[7];
	str[11]='/';
	str[12]=pesel[8];
	str[13]=pesel[9];
	str[14]='/';
	str[15]=pesel[10];
	str[16]='/';
	str[17]='\0';
	strcpy(path, pesel_base);
	strcat(path,"/");
	strcat(path,str);
	return 0;	
}



//tworzy katalogi sciezki PESEL
int make_dir_pesel(const char*pesel) {
	if(strlen(pesel)!=11) return -1;

	char str[20];
	char s[256];

	str[0]=pesel[0];
	str[1]=pesel[1];
	str[2]='\0';
	strcpy(s,pesel_base); //ustawia sciezke
	strcat(s,"/");
	strcat(s,str);	
	mkdir(s,0777 & ~022); // pierwszy kawalek
	str[2]='/';
	str[3]=pesel[2];
	str[4]=pesel[3];
	str[5]='\0';
	strcpy(s,pesel_base); //ustawia sciezke
	strcat(s,"/");
	strcat(s,str);
	mkdir(s,0777 & ~022); // drugi kawalek
	str[5]='/';
	str[6]=pesel[4];
	str[7]=pesel[5];
	str[8]='\0';
	strcpy(s,pesel_base); //ustawia sciezke
	strcat(s,"/");
	strcat(s,str);
	mkdir(s,0777 & ~022); // trzeci kawalek
	str[8]='/';
	str[9]=pesel[6];
	str[10]=pesel[7];
	str[11]='\0';
	strcpy(s,pesel_base); //ustawia sciezke
	strcat(s,"/");
	strcat(s,str);
	mkdir(s,0777 & ~022); // czwarty kawalek
	str[11]='/';
	str[12]=pesel[8];
	str[13]=pesel[9];
	str[14]='\0';
	strcpy(s,pesel_base); //ustawia sciezke
	strcat(s,"/");
	strcat(s,str);
	mkdir(s,0777 & ~022); // piaty kawalek
	str[14]='/';
	str[15]=pesel[10];
	str[16]='\0';
	strcpy(s,pesel_base); //ustawia sciezke
	strcat(s,"/");
	strcat(s,str);
	mkdir(s,0777 & ~022); // 6 kawalek
//printf("[]: %s\n",s);
	return 0;	
}



//zmienia uid --> /a/b/c/ (uzycie w UID32BASE oraz START)
void st_abc(const char*uid, char*abc) {
	*abc = '/';
	*(abc+1) = *uid;
	*(abc+2) = '/';
	*(abc+3) = *(uid+1);
	*(abc+4) = '/';
	*(abc+5) = *(uid+2);
	*(abc+6) = '/';
	*(abc+7) = '\0';
}
//--------------------------- END PATHS CREATION ------------------------------



//------------------------------- DATE-TIME -----------------------------------
//znakowanie czasu
//DATE=20170130-h:m:s&
int get_time(char*data) {
	char st[10];
	struct tm *ts;
	time_t tt;
	if(time(&tt)==-1) {strcpy(data,"NULL"); return -1;}
	//year
	ts=localtime(&tt);
	int x=1900+ts->tm_year;
	sprintf(st,"%d",x);
	strcpy(data,"DATE=");
	strcat(data,st);
	//month
	x=ts->tm_mon;x++; //poniwaz miesac liczny jest od 0..
	sprintf(st,"%d",x);
	if(strlen(st)==1) strcat(data,"0");
	strcat(data,st);
	//day
	x=ts->tm_mday;
	sprintf(st,"%d",x);
	if(strlen(st)==1) strcat(data,"0");
	strcat(data,st);
	strcat(data,"-");
	//hour
	x=ts->tm_hour;
	sprintf(st,"%d",x);
	if(strlen(st)==1) strcat(data,"0");
	strcat(data,st);
	strcat(data,":");
	//minuta
	x=ts->tm_min;
	sprintf(st,"%d",x);
	if(strlen(st)==1) strcat(data,"0");
	strcat(data,st);
	strcat(data,":");
	//sec
	x=ts->tm_sec;
	sprintf(st,"%d",x);
	if(strlen(st)==1) strcat(data,"0");
	strcat(data,st);
	strcat(data,"&");
	return 0;
}
//------------------------------ END DATE-TIME --------------------------------



//----------------------------- SAVE/READ FILE --------------------------------
//zapisuje linie do pliku tekstowego do pliku w modzie "a"
int add_text_line( const char*path, const char*fname, const char*str_line) {

	char st[512];

	strcpy(st, path);strcat(st,"/");strcat(st,fname);
	FILE*fd = fopen(st,"a");
	if(fd==NULL) return -1;
	  fprintf(fd,"%s\n",str_line);
	  fflush(fd);
	fclose(fd);
	return 0;
}


//dziala jak wyzejej, ale fname = path+fname
int add_string_to_file(const char*fname, const char*str_line) {
	FILE*fd = fopen(fname,"a");
	if(fd==NULL) return -1;
	  fprintf(fd, "%s\n", str_line);
	  fflush(fd);
	fclose(fd);
	return 0;
}
//-------------------------- END SAVE/READ FILE -------------------------------



//------------------------ STRIP DEPARTMENT STRING ----------------------------
//OP=register_node&UID32=null&EUID=null&NAME=..&
//        DEP=..&NIP=..&REGON=..&PUID=..&PORTP=..&PORTQ=..NODES=..&
//        sprawdza przez trzy dane NIP,REGON,DEP
//        nalezy wpisac UID32 =null&EUID=null&
int strip_dpt_string(const char*st,
	char*uid32, char*euid, char*name,
	char*dep, char*nip, char*regon, 
	char*puid, char*portp, char*portq, char*nodes) {

	int ch=1;

	select_value(st, "UID32", uid32);
	if(strcmp(uid32, "NULL") == 0) strcpy(uid32, "null");
	select_value(st, "EUID", euid);
	if(strcmp(euid, "NULL") == 0) strcpy(euid, "null");

	select_value(st, "NAME", name);
	if(strcmp(name, "NULL") == 0) {strcpy(name,"null"); ch=0;}
	select_value(st, "DEP", dep);
	if(strcmp(dep, "NULL") == 0) {strcpy(dep,"null"); ch=0;}
	select_value(st, "NIP", nip);
	if(strcmp(nip, "NULL") == 0) {strcpy(nip,"null"); ch=0;}
	select_value(st, "REGON", regon);
	if(strcmp(regon, "NULL") == 0) {strcpy(regon,"null"); ch=0;}
	select_value(st, "PUID", puid);
	if(strcmp(puid, "NULL") == 0) {strcpy(puid,"null"); ch=0;}
	select_value(st, "PORTP", portp);
	if(strcmp(portp, "NULL") == 0) {strcpy(portp,"null"); ch=0;}
	select_value(st, "PORTQ", portq);
	if(strcmp(portq, "NULL") == 0) {strcpy(portq,"null"); ch=0;}
	select_value(st, "NODES", nodes);
	if(strcmp(nodes, "NULL") == 0) {strcpy(nodes,"null"); ch=0;}


	//parsowanie qvectora
	if(strcmp(nip, "null") == 0) {
		if(strcmp(regon, "null") == 0) ch=0;
	}
	if(strcmp(puid,  "null") ==0) ch=0;
	if(strcmp(portq, "null") == 0) ch=0;
	if(strcmp(portp, "null") == 0) ch=0;
	if(strcmp(nodes, "null") == 0) ch=0;
	if(strcmp(dep,  "null") == 0) ch=0;
	if(strcmp(name, "null") == 0) ch=0;

	//parsowanie qversorow nip, regon (spelnia formule)
	if(strcmp(nip, "null") != 0) {
		if(!check_form_pesel(nip)) ch=0;
	}	

	if(strcmp(regon, "null") != 0) {
		if(!check_form_pesel(regon)) ch=0;
	}	
		
	return ch;
}



int strip_small_dpt_string(const char*st, char*nip, char*regon, char*dep ) {

	int ch = 1;

	select_value(st, "DEP", dep);
	if(strcmp(dep, "NULL") == 0) { strcpy(dep, "null"); ch=0; }

	select_value(st, "NIP", nip);
	if(strcmp(nip, "NULL") == 0) { strcpy(nip, "null"); ch=0; }

	select_value(st, "NIP", nip);
	if(strcmp(nip, "NULL") == 0) { strcpy(nip, "null"); ch=0; }

	select_value(st, "REGON", regon);
	if(strcmp(regon, "NULL") == 0) { strcpy(regon, "null"); ch=0;}

	//parsowanie qversorow nip, regon (spelnia formule)
	if(strcmp(nip, "null") != 0) {
		if(!check_form_pesel(nip)) ch=0;
	}	
	if(strcmp(regon, "null") != 0) {
		if(!check_form_pesel(regon)) ch=0;
	}
	return ch;
} 
//---------------------- END STRIP DEPARTMENT STRING --------------------------



//-------------------------- MAKE QVECTOR FOR OBJ -----------------------------
void create_dep_string(const char*uid32, const char*euid, const char*name,
	const char*dep, const char*nip, const char*regon, char*qstr) {

	char date[20]; 

	strcpy(qstr, "UID32=");
	strcat(qstr, uid32);
	strcat(qstr, "&EUID=");
	strcat(qstr, euid);
	strcat(qstr, "&NAME=");
	strcat(qstr, name);
	strcat(qstr, "&DEP=");
	strcat(qstr, dep);
	strcat(qstr, "&NIP=");
	strcat(qstr, nip);
	strcat(qstr, "&REGON=");
	strcat(qstr, regon);
	strcat(qstr, "&");
   
	if(get_time(date) == 0) {
		strcat(qstr, date);
	}
}



//sklada qvector dla NODE (DEP) i dodaje date
void create_dep_string_uid32base(const	char*uid32, const char*euid, 
	const char*dep, const char*name, const char*puid, const char*portp, 
	const char*portq, const char*nodes, char*qstr) {

	char date[20]; 

	strcpy(qstr, "UID32=");
	strcat(qstr, uid32);
	strcat(qstr, "&EUID=");
	strcat(qstr, euid);
	strcat(qstr, "&NAME=");
	strcat(qstr, name);
	strcat(qstr, "&DEP=");
	strcat(qstr, dep);
	strcat(qstr, "&PUID=");
	strcat(qstr, puid);
	strcat(qstr, "&PORTP=");
	strcat(qstr, portp);
	strcat(qstr, "&PORTQ=");
	strcat(qstr, portq);
	strcat(qstr, "&NODES=");
	strcat(qstr, nodes);
	strcat(qstr, "&");
   
	if(get_time(date) == 0) {
		strcat(qstr, date);
	}
}



//sklada qvector dla NODE (DEP) i dodaje date
void create_dep_string_full(const char*uid32, const char*euid, 
	const char*name, const char*dep, const char*nip, const char*regon, 
	const char*puid, const char*portp, 
	const char*portq, const char*nodes, char*qstr) {

	char date[20];   

	strcpy(qstr, "UID32=");
	strcat(qstr, uid32);
	strcat(qstr, "&EUID=");
	strcat(qstr, euid);
	strcat(qstr, "&NAME=");
	strcat(qstr, name);
	strcat(qstr, "&DEP=");
	strcat(qstr, dep);
	strcat(qstr, "&NIP=");
	strcat(qstr, nip);
	strcat(qstr, "&REGON=");
	strcat(qstr, regon);
	strcat(qstr, "&PUID=");
	strcat(qstr, puid);
	strcat(qstr, "&PORTP=");
	strcat(qstr, portp);
	strcat(qstr, "&PORTQ=");
	strcat(qstr, portq);
	strcat(qstr, "&NODES=");
	strcat(qstr, nodes);
	strcat(qstr, "&");
   
	if(get_time(date) == 0) {
		strcat(qstr, date);
	}
}
//------------------------ END MAKE QVECTOR FOR OBJ ---------------------------



//--------------------- CHECK DEP IN NIP OR REGON -----------------------------
//sprawdza czy dep jest juz zarejestrowany w NIP/REGON, ogolnie w jako
//OBJ w PESEL BASE
//dodatkowo pobiera qvector ze struktury dla OBJ
int check_dep_in_pesel_base(const char*dep, const char*nip, const char*regon,
										char*uid32, char*euid) {

	int ret;
	char path[512];
	char out[512];
	char old_dep[15];
	char old_nip[15];
	char old_regon[15];
	char *p;

	strcpy(uid32, "null");
	strcpy(euid, "null");


	//tworzy sciezke dostepu do struktury pesel dla nip
	ret = make_path_pesel(nip, path);
	if(ret < 0) return -1; //jak strlen(nip) != 11

	//tworzy nazwe pliku do odczytu i otwiera plik do odczytu
	strcat(path, "/");
	strcat(path, PESEL_NOD);

	//sprawdza dla nip
	FILE*fd = fopen(path, "r");
	if(fd == NULL) {
		//tworzy sciezke dostepu dla regon
		ret = make_path_pesel(regon, path);
		if(ret < 0) return -1; //jak strlen(regon) != 11

		//tworzy nazwe pliku do odczytu i otwiera plik do odczytu
		strcat(path, "/");
		strcat(path, PESEL_NOD);
		fd = fopen(path, "r");
		if (fd == NULL) return -1;
	}

	ret = 0;
	strcpy(out, "null");
	while(fgets(path, 512, fd) != NULL) {
		p = strchr(path, '\n');
		if(p != NULL) *p = '\0';
		p = strstr(path, dep);
		if(p != NULL) {strcpy(out, path); ret = 1;}
	}
	fclose(fd);

	if(ret) {
		//odmowa zamian jesli dep, nip/regon sa zgodne
		select_value(out, "NIP", old_nip);
		if(strcmp(old_nip, "NULL") == 0) strcpy(old_nip, "null");
		select_value(out, "REGON", old_regon);
		if(strcmp(old_regon, "NULL") == 0) strcpy(old_regon, "null");
		select_value(out, "DEP", old_dep);
		if(strcmp(old_dep, "NULL") == 0) strcpy(old_dep, "null");
		select_value(out, "UID32", uid32);
		if(strcmp(uid32, "NULL") == 0) strcpy(uid32, "null");
		select_value(out, "EUID", euid);
		if(strcmp(euid, "NULL") == 0) strcpy(euid, "null");

		if(strcmp(old_dep, dep) == 0 && 
					 strcmp(old_nip, nip) == 0  &&
					 strcmp(uid32, "null") != 0 &&
					 strcmp(euid, "null") != 0) return 1;
		if(strcmp(old_dep, dep) == 0 && 
					strcmp(old_regon, regon) == 0 &&
					strcmp(nip, "null") == 0 &&
					strcmp(uid32, "null") != 0 &&
					strcmp(euid, "null") != 0) return 1;			
	}
	else {
		//jesli ma być zarejestowany nowy dep dla tych samych nip lub regon
		strcpy(out, path);
		select_value(out, "NIP", old_nip);
		if(strcmp(old_nip, "NULL") == 0) strcpy(old_nip, "null");
		select_value(out, "UID32", uid32);
		if(strcmp(uid32, "NULL") == 0) strcpy(uid32, "null");
		select_value(out, "EUID", euid);
		if(strcmp(euid, "NULL") == 0) strcpy(euid, "null");
	
		if(strcmp(uid32, "null") == 0) return 0;
		if(strcmp(euid, "null") == 0) return 0;

		if(strcmp(old_nip, nip) == 0 &&
		  			strcmp(uid32, "null") != 0 &&
		  			strcmp(euid, "null") != 0) return 1;

		if(strcmp(old_regon, regon) == 0 &&
		  			strcmp(uid32, "null") != 0 &&
		  			strcmp(euid, "null") != 0) return 1;
	}



	return 0;
}
//---------------------- CHECK DEP IN NIP OR REGON ----------------------------



//------------------------- ADD OBJ TO PESEL STRUCT ---------------------------
//dodaje qvector NODE do struktury PESEL 
int add_node_to_pesel(const char*pesel, const char*qstr) {

	int ret;
	char path[128];

	//buduje strukture katalogow PESEL
	ret =  make_dir_pesel(pesel);
	if(ret < 0) return -1;


	//tworzy sciezke dostpu struktury pesel
	ret = make_path_pesel(pesel, path);
	if(ret < 0) return -1; 


	ret = add_text_line(path, PESEL_NOD, qstr);

	return ret;
}
//----------------------- END ADD OBJ TO PESEL STRUCT -------------------------



//------------------------- ADD OBJ TO UID32BASE ------------------------------
//dodaje qvector NODE to UID32BASE
int add_node_to_uid32base(const char*uid32, const char*str) {

	int ret;
	char path[128];
	char abc[15];

	//fragment path uidp /X/X/X/ 
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	//tworzy katalog o nazwie user_qvector.uidp
	mkdir(path, 0777 & ~022); // drugi kawalek
	strcat(path, "/");
	strcat(path, QNODE);

	//zapis do UID32BASE
	ret = add_string_to_file(path, str);

	return ret;
}

//dodaje qvector NODE to UID32BASE TMP
int add_node_to_uid32base_tmp(const char*uid32, const char*str) {

	int ret;
	char path[128];
	char abc[15];

	//fragment path uidp /X/X/X/ 
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	//tworzy katalog o nazwie user_qvector.uidp
	mkdir(path, 0777 & ~022); // drugi kawalek
	strcat(path, "/");
	strcat(path, "TMP");

	//zapis do UID32BASE
	ret = add_string_to_file(path, str);

	return ret;
}

//kopiowanie tmp na uid32 base
void copy_tmp_to_uid32base(const char*uid32) {

	char path[128];
	char path_tmp[128];
	char abc[15];
	char st[256];

	// path uid32 /X/X/X/ fo pliku tmp
	st_abc(uid32, abc);
	strcpy(path_tmp, uid32base);
	strcat(path_tmp, abc);
	strcat(path_tmp, uid32);
	strcat(path_tmp, "/");
	strcpy(path, path_tmp);
	strcat(path_tmp, "TMP");

	//dodaje nazwe QNODE do path
	strcat(path, QNODE);

	strcpy(st, "cp ");
	strcat(st, path_tmp);
	strcat(st, " ");
	strcat(st, path);
	system(st);

	//usuwa plik TMP
	remove(path_tmp);
}
//---------------------- END ADD OBJ TO UID32BASE -----------------------------



struct dep_full_ {
	char uid32[35];
	char euid[35];
	char dep[15];
	char name[80];
	char nip[15];
	char regon[15];
	char puid[10];
	char portp[10];
	char portq[10];
	char nodes[6];
}; // depfull;



//----------------------- CHANGE EUID IN PESEL BASE --------------------------- 
//zmienia euid in pesel base
int chg_euid_in_pesel_base(const char*dep, const char*nip, 
									const char*regon, const char*euid, char*uid32) {
		
	struct dep_full_ depfull;

	int ret;
	char path[512];
	char st[512];
	char str[256];
	char *p;

	//--- zgloszenie przez NIP (zmienia wszystkie DEP) ---
	//jesli dep=null& to blokuje wszytkie departamenty
	if(strcmp(nip, "null") != 0) {
		//tworzy sciezke dostepu do struktury pesel dla nip
		ret = make_path_pesel(nip, st);
		if(ret < 0) return -1; //jak strlen(nip) != 11

		//tworzy nazwe pliku do odczytu i otwiera plik do odczytu
		strcpy(path, st);
		strcat(st, PESEL_NOD);

		//otwiera dla nip
		FILE*fd = fopen(st, "r");
		if(fd == NULL) return -1;

		while(fgets(st, 512, fd) != NULL) {

			p = strchr(st, '\n');
			if(p != NULL) *p = '\0';

			//rozklada linie zawierajace wskazany nip
			strip_dpt_string(st, depfull.uid32, depfull.euid, depfull.name,
					depfull.dep, depfull.nip, depfull.regon, 
					depfull.puid, depfull.portp, depfull.portq, depfull.nodes);
					strcpy(uid32, depfull.uid32);

			//jesli dep="null" to zmienia wszystkie departamenty
			if(strcmp(dep, "null") == 0) {
				//wymienia euid (blokowanie trwale)
				strcpy(depfull.euid, euid);
				//sklada ponownie 
				create_dep_string(depfull.uid32, depfull.euid, depfull.name, 
					depfull.dep, depfull.nip, depfull.regon, st);
				//zapis do LIST_PESEL_NOD (backup)
				add_text_line(list_base, LIST_PESEL_NOD, st);
			} else {
				//zmienia tylko jesli dep == depfull.dep
				if(strcmp(dep, depfull.dep) == 0) {
					//wymienia euid (blokowanie trwale)
					strcpy(depfull.euid, euid);
					//sklada ponownie 
					create_dep_string(depfull.uid32, depfull.euid, depfull.name, 
						depfull.dep, depfull.nip, depfull.regon, st);
					//zapis do LIST_PESEL_NOD (backup)
					add_text_line(list_base, LIST_PESEL_NOD, st);
				}
			}

			//zapisuje do pliku tmp
			if(add_text_line(path, "tmp.q", st) < 0) {
				fclose(fd);
				return -1;
			}
		}
		fclose(fd);

		//zamienia pliki w koncie nip
		strcpy(st, path);
		strcat(path, PESEL_NOD);
		strcat(st,  "tmp.q");
		remove(path);
		rename(st, path);

		//jesli regon != 'null"
		if(strcmp(depfull.regon, "null") != 0) {

			ret = make_path_pesel(regon, st);
			if(ret < 0) return -1; //jak strlen(nip) != 11
			strcat(st, PESEL_NOD);

			strcpy(str, "cp ");
			strcat(str, path);
			strcat(str, " ");
			strcat(str, st);
			fd = popen(str, "r");
			if(fd == NULL) return -1;
			strcpy(st, "null");
			fgets(st, 512, fd);
			pclose(fd); 
			if(strcmp(st, "null") == 0) return -1;
		}
		return 0;
	}
	

	//--- zgloszenie przez REGON (zmienia wszystkie DEP) ---
	//jesli dep=null& to blokuje wszytkie departamenty
	if(strcmp(regon, "null") != 0) {
		//tworzy sciezke dostepu do struktury pesel dla nip
		ret = make_path_pesel(regon, st);
		if(ret < 0) return -1; //jak strlen(nip) != 11

		//tworzy nazwe pliku do odczytu i otwiera plik do odczytu
		strcpy(path, st);
		strcat(st, PESEL_NOD);

		//otwiera dla regon
		FILE*fd = fopen(st, "r");
		if(fd == NULL) return -1;

		while(fgets(st, 512, fd) != NULL) {
			p = strchr(st, '\n');
			if(p != NULL) *p = '\0';

			//rozklada linie zawierajace wskazany regon
			strip_dpt_string(st, depfull.uid32, depfull.euid, depfull.name,
					depfull.dep, depfull.nip, depfull.regon, 
					depfull.puid, depfull.portp, depfull.portq, depfull.nodes);
					strcpy(uid32, depfull.uid32);

			//jesli dep="null" to zmienia wszystkie departamenty
			if(strcmp(dep, "null") == 0) {
				//wymienia euid (blokowanie trwale)
				strcpy(depfull.euid, euid);
				//sklada ponownie 
				create_dep_string(depfull.uid32, depfull.euid, depfull.name, 
					depfull.dep, depfull.nip, depfull.regon, st);
				//zapis do LIST_PESEL_NOD (backup)
				add_text_line(list_base, LIST_PESEL_NOD, st);
			} else {
				//zmienia tylko jesli dep == depfull.dep
				if(strcmp(dep, depfull.dep) == 0) {
					//wymienia euid (blokowanie trwale)
					strcpy(depfull.euid, euid);
					//sklada ponownie 
					create_dep_string(depfull.uid32, depfull.euid, depfull.name, 
						depfull.dep, depfull.nip, depfull.regon, st);
				//zapis do LIST_PESEL_NOD (backup)
				add_text_line(list_base, LIST_PESEL_NOD, st);
				}
			}

			//zapisuje do pliku tmp
			if(add_text_line(path, "tmp.q", st) < 0) {
				fclose(fd);
				return -1;
			}
		}
		fclose(fd);

		//zamienia pliki w koncie regon
		strcpy(st, path);
		strcat(path, PESEL_NOD);
		strcat(st,  "tmp.q");
		remove(path);
		rename(st, path);

		//jesli nip != 'null"
		if(strcmp(depfull.nip, "null") != 0) {
			ret = make_path_pesel(nip, st);
			if(ret < 0) return -1; //jak strlen(nip) != 11
			strcat(st, PESEL_NOD);

			strcpy(str, "cp ");
			strcat(str, path);
			strcat(str, " ");
			strcat(str, st);
			fd = popen(str, "r");
			if(fd == NULL) return -1;
			strcpy(st, "null");
			fgets(st, 512, fd);
			pclose(fd); 
			if(strcmp(st, "null") == 0) return -1;
		}
		return 0;
	}


	return -1; //jesli nip=="null" && regon=="null"
}
//--------------------- END CHANGE EUID IN PESEL BASE -------------------------



//----------------------- CHANGE EUID IN UID32BASE ---------------------------- 
//zmienia euid w UID32BASE
int chg_euid_in_uid32base(const char*dep, const char*euid, const char*uid32) {

	struct dep_full_ depfull;

	int ret;
	int port;
	char path[512];
	char st[512];
	char send[128];
	char reply[128];
	char abc[15];
	char ipUIDL[20];
	char *p;
 	FILE*fd;

   //pobiera adress IP NODE z pliku
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, IPNODE); //odczytuje plik ip.q
	fd = fopen(path, "r");
	if(fd == NULL) return -1;
	fgets(st, 20, fd);
	fclose(fd);
	//IPADDR=..&
	select_value(st, "IPADDR", ipUIDL);
	if(strcmp(ipUIDL, "NULL") == 0) return -1; 

	// path uidp /X/X/X/ do odczytu linii 
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, QNODE); //plik node.q

	//blokuje wszystkie departamenty
	if(strcmp(dep, "null") == 0) {

		fd = fopen(path, "r");
		if(fd == NULL) return -1;

		while(fgets(st, 512, fd) != NULL) {

			p = strchr(st, '\n');
			if(p != NULL) *p = '\0';

			strip_dpt_string(st, depfull.uid32, depfull.euid, depfull.name,
					depfull.dep, depfull.nip, depfull.regon, 
					depfull.puid, depfull.portp, depfull.portq, depfull.nodes);

			//zmiena euid
			if(strcmp(uid32, depfull.uid32) == 0) {
				create_dep_string_uid32base(uid32, euid, depfull.dep, depfull.name, 
										 		 depfull.puid, depfull.portp, 
												 depfull.portq, depfull.nodes, st);

				// --- blokowanie w UIDL ---
				//-----------------------------------------------------------------
				port = atoi(depfull.puid);
				if(port == 0) {
					  fclose(fd);
					  return -1;
				} 
				strcpy(send, "OP=block_department&DEP=");
				strcat(send, depfull.dep);
				strcat(send, "&EUID=");
				strcat(send, euid);
				strcat(send, "&");

				ret = client_U(ipUIDL, port, send, reply);
				if(ret < 0) { fclose(fd); return -1;}
				p = strstr(reply, "refused");
				if(p != NULL) { fclose(fd); return -1;}
				//-----------------------------------------------------------------
 
				//zapis do UID32BASE tmp
				ret = add_node_to_uid32base_tmp(uid32, st);
				if(ret < 0) {fclose(fd); return -1;}
				//zapis do LIST_UID32BASE (backup)
				add_text_line(list_base, LIST_UID32BASE, st);
			}
		}
		fclose(fd);	

		//kopiuje tmp na QNODE i usuwa TMP
		copy_tmp_to_uid32base(uid32);

		return 0;
	}



	//blokuje wybrany departament
	if(strcmp(dep, "null") != 0) {

		fd = fopen(path, "r");
		if(fd == NULL) return -1;

		while(fgets(st, 512, fd) != NULL) {

			p = strchr(st, '\n');
			if(p != NULL) *p = '\0';

			strip_dpt_string(st, depfull.uid32, depfull.euid, depfull.name,
					depfull.dep, depfull.nip, depfull.regon, 
					depfull.puid, depfull.portp, depfull.portq, depfull.nodes);

			//zmiena euid
			if(strcmp(uid32, depfull.uid32) == 0 && 
				strcmp(dep, depfull.dep) == 0 ) {
				create_dep_string_uid32base(uid32, euid, depfull.dep, depfull.name, 
										 		 depfull.puid, depfull.portp, 
												 depfull.portq, depfull.nodes, st);
 

				// --- blokowanie w UIDL ---
				// ----------------------------------------------------------------
				port = atoi(depfull.puid);
				if(port == 0) {
					  fclose(fd);
					  return -1;
				} 
				strcpy(send, "OP=block_department&DEP=");
				strcat(send, depfull.dep);
				strcat(send, "&EUID=");
				strcat(send, euid);
				strcat(send, "&");

				ret = client_U(ipUIDL, port, send, reply);
				if(ret < 0) { fclose(fd); return -1;}
				p = strstr(reply, "refused");
				if(p != NULL) { fclose(fd); return -1;}
				// ----------------------------------------------------------------


				//zapis do UID32BASE
				ret = add_node_to_uid32base_tmp(uid32, st);
				if(ret < 0) {fclose(fd); return -1;}
				//zapis do LIST_UID32BASE (backup)
				add_text_line(list_base, LIST_UID32BASE, st);
			}
		}
		fclose(fd);	

		//kopiuje tmp na QNODE i usuwa TMP
		copy_tmp_to_uid32base(uid32);

		return 0;
	}

	return -1; 
}
//--------------------- END CHANGE EUID IN UID32BASE BASE ---------------------



//---------------------------- GET NODE TECHNICAL DATA ------------------------
int read_dep_data_in_UID32BASE(const char*dep, const char*uid32, char*qstr) {

	struct dep_full_ depfull;

	char path[512];
	char st[512];
	char abc[15];
	char ip[20];
	char date[20];
	char *p;
 	FILE*fd;

	strcpy(qstr, "null");

	if(strcmp(dep, "null") == 0) return -1;
	if(strcmp(uid32, "null") == 0) return -1;

	//pobiera adress IP NODE z pliku
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, IPNODE); //odczytuje plik ip.q
	fd = fopen(path, "r");
	if(fd == NULL) return -1;
	fgets(st, 20, fd);
	fclose(fd);
	//IPADDR=..&
	select_value(st, "IPADDR", ip);
	if(strcmp(ip, "NULL") == 0) return -1;

	// path uidp /X/X/X/ do odczytu linii 
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, QNODE); //plik node.q

	fd = fopen(path, "r");
	if(fd == NULL) return -1;

	strcpy(path, "null");
	while(fgets(st, 512, fd) != NULL) {
		p = strchr(st, '\n');
		if(p != NULL) *p = '\0';

		p = strstr(st, dep);
		if(p != NULL) strcpy(path, st);
	}

	fclose(fd);
	if(strcmp(path, "null") == 0) return -1;

	strip_dpt_string(path, depfull.uid32, depfull.euid, depfull.name,
					depfull.dep, depfull.nip, depfull.regon, 
					depfull.puid, depfull.portp, depfull.portq, depfull.nodes);

	//parsowanie odczytanej linii
	if(strcmp(depfull.uid32, "null") == 0) return -1;
	if(strcmp(depfull.euid, "null") == 0)  return -1;
	if(strcmp(depfull.puid, "null") == 0)  return -1;
	if(strcmp(depfull.portp, "null") == 0) return -1;
	if(strcmp(depfull.portq, "null") == 0) return -1;
	if(strcmp(depfull.nodes, "null") == 0) return -1;

	select_value(path, "DATE", date);
	if(strcmp(date, "NULL") == 0) strcpy(date, "null");

	//buduje qstr
	strcpy(qstr, "UID32=");

	strcat(qstr, depfull.uid32);
	strcat(qstr, "&EUID=");
	strcat(qstr, depfull.euid);
	strcat(qstr, "&DEP=");
	strcat(qstr, depfull.dep);
	strcat(qstr, "&LIP=");
	strcat(qstr, ip);
	strcat(qstr, "&PUID=");
	strcat(qstr, depfull.puid);
	strcat(qstr, "&PORTP=");
	strcat(qstr, depfull.portp);
	strcat(qstr, "&PORTQ=");
	strcat(qstr, depfull.portq);
	strcat(qstr, "&NODES=");
	strcat(qstr, depfull.nodes);

	if(strcmp(date, "null") != 0) {
		strcat(qstr, "&DATE=");
		strcat(qstr, date);
	} 

	strcat(qstr, "&");

	return 0;
} 
//-------------------------- END GET NODE TECHNICAL DATA ----------------------




//---------------------------------- USER/ADMIN -------------------------------
struct userfull {
	char uid32[35];
	char euid[35];
	char uidp[35];
	char euidp[35];
	
	char pesel[15];
	char dep[15];
	char name[80];
	char statt[10];
 	char range[15];
	char act[5];
}; 




//sprawdza input string usera/admina
int strip_user_string(const char*qstr,
	char*uid32, char*euid, char*uidp, char*euidp,char*pesel,
	char*dep, char*name, char*statt, char*range, char*act) {

	int ret = 1;

	select_value(qstr, "UID32", uid32);
	if(strcmp(uid32, "NULL") == 0) { strcpy(uid32, "null"); ret = 0; }
	select_value(qstr, "EUID", euid);
	if(strcmp(euid, "NULL") == 0) { strcpy(euid, "null"); ret = 0; }
	select_value(qstr, "UIDP", uidp);
	if(strcmp(uidp, "NULL") == 0) { strcpy(uidp, "null"); ret = 0; }
	select_value(qstr, "EUIDP", euidp);
	if(strcmp(euidp, "NULL") == 0) { strcpy(euidp, "null"); ret = 0; }
	select_value(qstr, "DEP", dep);
	if(strcmp(dep, "NULL") == 0) { strcpy(dep, "null"); ret = 0; }
	select_value(qstr, "NAME", name);
	if(strcmp(name, "NULL") == 0) { strcpy(name, "null"); ret = 0; }
	select_value(qstr, "STAT", statt);
	if(strcmp(statt, "NULL") == 0) { strcpy(statt, "null"); ret = 0; }
	select_value(qstr, "RANGE", range);
	if(strcmp(range, "NULL") == 0) { strcpy(range, "null"); }
	select_value(qstr, "ACT", act);
	if(strcmp(act, "NULL") == 0) { strcpy(act, "null"); }
	select_value(qstr, "PESEL", pesel);
	if(strcmp(pesel, "NULL") == 0) { strcpy(pesel, "null"); ret = 0; }

	//sprawdza pesel czy spelnaia formule
	if(!check_form_pesel(pesel)) ret = 0; 

	return ret;
}



//tworzy peny string dla przeslania zwrotnego do UIDL
void create_user_qstring_full( const char*uid32, const char*euid, 
			const char*uidp, const char*euidp, const char*pesel,
			const char*dep, const char*name, const char*statt, const char*range, 
			const char*act, char*qstr) {

	char date[20];

	strcpy(qstr, "UID32=");
	strcat(qstr, uid32);
	strcat(qstr, "&EUID=");
	strcat(qstr, euid);
	strcat(qstr, "&UIDP=");
	strcat(qstr, uidp);
	strcat(qstr, "&EUIDP=");
	strcat(qstr, euidp);
	strcat(qstr, "&PESEL=");
	strcat(qstr, pesel);
	strcat(qstr, "&DEP=");
	strcat(qstr, dep);
	strcat(qstr, "&NAME=");
	strcat(qstr, name);
	strcat(qstr, "&STAT=");
	strcat(qstr, statt);
	strcat(qstr, "&RANGE=");
	strcat(qstr, range);
	strcat(qstr, "&ACT=");
	strcat(qstr, act);
	strcat(qstr, "&");

	if(get_time(date) == 0) strcat(qstr, date);

}



void create_user_small_string( const char*uid32, const char*euid, 
			const char*uidp, const char*euidp, const char*pesel,
			const char*dep, const char*name, const char*statt,  
		   char*qstr) {

	char date[20];

	strcpy(qstr, "UID32=");
	strcat(qstr, uid32);
	strcat(qstr, "&EUID=");
	strcat(qstr, euid);
	strcat(qstr, "&UIDP=");
	strcat(qstr, uidp);
	strcat(qstr, "&EUIDP=");
	strcat(qstr, euidp);
	strcat(qstr, "&PESEL=");
	strcat(qstr, pesel);
	strcat(qstr, "&DEP=");
	strcat(qstr, dep);
	strcat(qstr, "&NAME=");
	strcat(qstr, name);
	strcat(qstr, "&STAT=");
	strcat(qstr, statt);
	strcat(qstr, "&");

	if(get_time(date) == 0) strcat(qstr, date);
}



//sprawdza pesel usera w bazie PESEL
int check_user_pesel(const char*pesel, char*out) {

	int ret;
	char path[512];
	char *p;

	strcpy(out, "null");

	ret = make_path_pesel(pesel, path);
	if(ret < 0) return -2; //jesli strlen(pesel) != 11	

	//dodaje nazwe pliku zawierajcego dane usera
	strcat(path, PESEL_USR);

	FILE*fd = fopen(path, "r");
	if(fd == NULL) return -1;

	while(fgets(path, 512, fd) != NULL); //pobiera ostatni wpis

	fclose(fd);

	p = strchr(path, '\n');
	if(p != NULL) *p = '\0';	

	strcpy(out, path);

	return 0;
}



//zapis danych uzytkownika do struktur 
int save_user_to_structures(const char*uidp, const char*pesel, 
									 const char*qstr) {

	int ret;
	char path[256];
	char abc[15];

	//--- zapis do UD32BASE (uidp) ---
	st_abc(uidp, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uidp);
	mkdir(path, 0777 & ~022); // dodaje uidp do path
	strcat(path, "/");
	strcat(path, QUSR); //plik usr.q
	ret =  add_string_to_file(path, qstr);
	if(ret < 0) return -1;


	//--- zapis do PESEL base ---
	ret = make_dir_pesel(pesel); //tworzy strukture katalogow bazy PESEL 
	if(ret < 0) return -1; //jesli strlen(pesel) != 11 (malo prawdopodobne)

	//tworzy sciezke dostepu do pliku
	ret = make_path_pesel(pesel, path);
	if(ret < 0) return -1; //jesli strlen(pesel) != 11 (malo prawdopodobne)
	ret = add_text_line(path, PESEL_USR, qstr);
	if(ret < 0) return -1;

	// --- zapis do list backup ---
	add_text_line(list_base, LIST_PESEL_USR, qstr);
	add_text_line(list_base, LIST_UID32BASE, qstr);

	return 0;
}



//sprawdza usera w bazie UID32BASE
int check_user_in_uid32base(const char*uid32, const char*euid, 
									 const char*uidp, const char*euidp) {
	int ret;
	char path[512];
	char abc[15];
	char b_uid32[35];
	char b_euid[35];
	char b_uidp[35];
	char b_euidp[35];
	FILE*fd;

	//sprawdza NODE czy nie jest zablokowany
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, QNODE); //plik node.q
	fd = fopen(path, "r");
	if(fd == NULL) return -1;
	while(fgets(path, 512, fd) != NULL); //odczyt ostatniego wpisu
	fclose(fd);

	select_value(path, "UID32", b_uid32);
	select_value(path, "EUID", b_euid);
	//sprawdza czy uid nie zostaly zmienione w node.q
	if(strcmp(b_uid32, uid32) != 0)  return -1;
	if(strcmp(b_euid,  euid) != 0)   return -1; 

	//sprawdza czy user nie jest zablokowany
	st_abc(uidp, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uidp);
	strcat(path, "/");
	strcat(path, QUSR); //plik usr.q
	fd = fopen(path, "r");
	if(fd == NULL) return -1;
	while(fgets(path, 512, fd) != NULL); //odczyt ostatniego wpisu
	fclose(fd);

	select_value(path, "UID32", b_uid32);
	select_value(path, "EUID",  b_euid);
	select_value(path, "UIDP",  b_uidp);
	select_value(path, "EUIDP", b_euidp);

	ret = 0;
	//sprawdza czy uid nie zostaly zmienione w usr.q
	if(strcmp(b_uid32, uid32) != 0)   ret = -1;
	if(strcmp(b_euid,  euid)  != 0)   ret = -1; 	
	if(strcmp(b_uidp,  uidp)  != 0)   ret = -2;
	if(strcmp(b_euidp, euidp) != 0)   ret = -2; 

	return ret;
}



//odczytuje q-string z bazy PESEL dla usera
int read_user_from_pesel(const char*pesel, char*qstr) {
	
	int ret;
	char path[512];
	FILE*fd;

	//tworzy sciezke dostepu do pliku
	ret = make_path_pesel(pesel, path);
	if(ret < 0) return -1; //jesli pesel nie zawiera 11 znakow
	strcat(path, "/");
	strcat(path, PESEL_USR);

	fd = fopen(path, "r");
	if(fd == NULL) return -1;

	while(fgets(qstr, 512, fd) != NULL); //wczytuje ostani wpis
	fclose(fd);


	return 0;
}



//wysyla poecenie blokowania trwalego uzytkownika w UIDL
int send_block_usr_to_UIDL(const char*uid32, const char*pesel,
					 const char*euidp) {

	int ret;
	int port;
	char path[512];
	char abc[15];
	char portUIDL[10];
	char ip[20];
	char send[80];
	char *p;
	FILE*fd; 

	//pobiera dane node
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, QNODE); //plik node.q
	fd = fopen(path, "r");
	if(fd == NULL) return -1;
	while(fgets(path, 512, fd) != NULL); //odczyt ostatniego wpisu
	fclose(fd);

	select_value(path, "PUID", portUIDL);
	if(strcmp(portUIDL, "NULL") == 0) return -1;

	//pobiera node IP adres
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, IPNODE); //plik ip.q
	fd = fopen(path, "r");
	if(fd == NULL) return -1;
	while(fgets(path, 512, fd) != NULL); //odczyt ostatniego wpisu
	fclose(fd);

	select_value(path, "IPADDR", ip);
	if(strcmp(ip, "NULL") == 0) return -1;
	
	strcpy(send, "OP=block_user_by_UIDC&PESEL=");
	strcat(send, pesel);
	strcat(send, "&EUIDP=");
	strcat(send, euidp);
	strcat(send, "&");

	port  = atoi(portUIDL);
	if(port == 0) return -1;

	//printf("\n\nIP: %s, PORT: %d\n\n", ip, port);

	// --- wysyla polecenie do UIDL ---
	ret = client_U(ip, port, send, path);
	if(ret < 0) return -1;
	p = strstr(path, "refused");
	if(p != NULL) return -1;

	return 0;
}
//------------------------------- END USER/ADMIN ------------------------------



//----------------------------------- MAP PESEL -------------------------------
//zpis qstr OBJ do struktur
int save_pesel_obj(const char*pesel, const char*qstr) {

	int ret;
	char path[256];

	//--- zapis do PESEL base ---
	ret = make_dir_pesel(pesel); //tworzy strukture katalogow bazy PESEL 
	if(ret < 0) return -1; //jesli strlen(pesel) != 11 (malo prawdopodobne)

	//tworzy sciezke dostepu do pliku
	ret = make_path_pesel(pesel, path);
	if(ret < 0) return -1; //jesli strlen(pesel) != 11 (malo prawdopodobne)
	ret = add_text_line(path, PESEL_OBJ, qstr);
	if(ret < 0) return -1;

	// --- zapis do list backup ---
	add_text_line(list_base, LIST_PESEL_OBJ, qstr);

	return 0;
}



//odczytuje liczbe wpisow dla pesel OBJ
int get_nlines_obj(const char*pesel) {

	int ret;
	char path[256];

	//tworzy sciezke dostepu do pliku
	ret = make_path_pesel(pesel, path);
	if(ret < 0) return 0; //jesli strlen(pesel) != 11 (malo prawdopodobne)
	strcat(path, PESEL_OBJ);

	FILE*fd = fopen(path, "r");
	if(fd == NULL) return 0;

	ret=0;
	while(fgets(path, 256, fd) != NULL) {
		ret++;
	}
	fclose(fd);

	return ret;
}



//zwraca linie pesel OBJ o okreslonym numerze
int read_obj_pesel_line(const char*pesel, int nline, char*out) {

	int ret;
	char path[256];
	char *p;

	//tworzy sciezke dostepu do pliku
	ret = make_path_pesel(pesel, path);
	if(ret < 0) return -1; //jesli strlen(pesel) != 11 (malo prawdopodobne)
	strcat(path, PESEL_OBJ);

	FILE*fd = fopen(path, "r");
	if(fd == NULL) return -1;

	ret=0;
	while(fgets(path, 256, fd) != NULL) {
		ret++;
		if(ret == nline) { 
			fclose(fd); 
			p = strchr(path, '\n');
			if(p != NULL) *p = '\0';
			strcpy(out, path);
			return 0; //jesli znalazl szukana nline
		} 
	}
	fclose(fd);

	return -1;
}



//pobiera dane node 
int get_node_data(const char*uid32, const char*dep, char*out) {

	char path[265];
	char qstr[512];
	char ip[20];
	char abc[15];
	char *p;
	FILE*fd;

	strcpy(qstr, "null");

	//--- odczyt z UD32BASE (uid32) danych node---
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, QNODE); //plik node.q
	fd = fopen(path, "r");
	if(fd == NULL) return -1;
	while(fgets(path, 512, fd) != NULL) { 
		p = strstr(path, dep);
		if(p != NULL) strcpy(qstr, path);
	}
	fclose(fd);
	p = strchr(qstr, '\n');
	if(p != NULL) *p = '\0';

	//sprawdza czy znalazl okreslona w DEP linie
	if(strcmp(qstr, "null") == 0) {
		strcpy(out, "null");
		return -1;
	}

	//--- odczyt z UD32BASE (uid32) adresu ip---
	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, IPNODE); //plik ip.q
	fd = fopen(path, "r");
	if(fd == NULL) return -2;
	fgets(path, 512, fd); //pobiera adres IP NODE
	fclose(fd);

	//(IPADDR=..&)
	select_value(path, "IPADDR", ip);
	if(strcmp(ip, "NULL") == 0) return -2;

	strcpy(out, "LIP=");
	strcat(out, ip);
	strcat(out, "&");
	strcat(out, qstr);

	return 0;
}
//--------------------------------- END MAP OBJ -------------------------------



//----------------------------- (static/var) IP node --------------------------
int check_ip_form(const char*ip) {
	
	//xxx.xxx.xxx.xxx
	int i, n;

	n = 0;

	//sprawdza liczbe "."
	for(i=0; i < (int)strlen(ip); i++) {
		if(*(ip + i) == '.') n++;
	}

	if(n != 3) return 0;
	
	n = 0;
	//sprawdza czy sa to cyfry i kropki
	for(i=0; i < (int)strlen(ip); i++) {
		if( (*(ip + i) - 48) > 9 && (*(ip + i) - 48) < 0 )  n++;
	}
	if(n != 0) return 0;

	return 1;
}


//backup ip (static/var) 
//( nie robi stosu histori zmian adresow ip przychodzacych z nodes)
int ip_backup(const char*uid32, const char*ip) {

	char st[80];
	char qstr[80];
	char str[80];
	char path[128];
	int i;
	int ch;
	long j;
   long fsize;
   long nlines;	

	//buduje qstring (UID32=..&IPADDR=..&)
	strcpy(str, "UID32=");
	strcat(str, uid32);
	strcat(str, "&IPADDR=");
	strcat(str, ip);
	strcat(str, "&");
	memset(qstr, '0', 80);
	memcpy(qstr, str, (size_t)strlen(str)+1); //string wsadzony razem z '\0'

	//tworzy sciezke dostepu do pliku do nadpisania
	strcpy(path, list_base);
	strcat(path, "/");
	strcat(path, LAST_IPNODE);
	
 	ch = 0;
   FILE*fw = fopen(path, "r+");
		if(fw == NULL) goto label_end;
   	fsize = fseek(fw, 0L, SEEK_END);
    	fsize = ftell(fw);
    	fseek(fw, 0L, SEEK_SET);
    	nlines = fsize/65;

   for(i=0; i < (int)nlines; i++) {
		fread(str, 65, 1, fw); 
		select_value(str, "UID32", st);
		if(strcmp(st, uid32) == 0)  {
			j = (long)i * 65;
			fseek(fw, j, SEEK_SET);
			fwrite(qstr, 65, 1, fw);
			fflush(fw);
			fclose(fw);
			return 1;		//znalazl, zmienil i wyskauje
		}	 
   }
	fclose(fw);    

label_end:

	//jak nie bylo znalezioneg uid32 to dopisuje do pliku
	fw = fopen(path, "a");
	if(fw == NULL) return -1;
	if(!ch) {
		fwrite(	qstr, 65, 1, fw);
		fflush(fw);
	}
	fclose(fw);

	return 1;
}



//zapisuje adres IP (WAN) wezla UIDL 
int save_ip_addr(const char*uid32, const char*ip) {

	char qstr[30];
	char path[265];
	char abc[15];
	FILE*fd;

	strcpy(qstr, "IPADDR=");
	strcat(qstr, ip);
	strcat(qstr, "&"); 

	st_abc(uid32, abc);
	strcpy(path, uid32base);
	strcat(path, abc);
	strcat(path, uid32);
	strcat(path, "/");
	strcat(path, IPNODE); //plik ip.q
	fd = fopen(path, "w");
	if(fd == NULL) return -1;
	fprintf(fd, "%s\n", qstr);
	fclose(fd);

	//backup 
	ip_backup(uid32, ip);

	return 0;
}
//-------------------------- END (static/var) IP node -------------------------




//---------------------------------- CLEAR BASE -------------------------------
//czysci UID32BASE
void clear_UID32BASE(void) {
	char str[126]="";
	char begin[126];
	char beg1[126];
	char beg2[126];
	char st[126];
	char *p;
	char let[3];
	int i,j,k;
	//FILE*fp;
		
	//budowanie dostepu do UID32BASE	
	strcat(str,uid32base);		 
	strcpy(begin,str);
   umask(0);
	mkdir(str,0777);
	strcat(str, "/name");
	remove(str); //usuwa poprzedni zapis jesli byl
	add_string_to_file(str, "UID32BASE"); //zapisuje nazwe bazy

	//pierwsza seria katalogow .../UID32BASE/x
	for(i=65;i<91;i++) {
		strcpy(str,begin);
		let[0]=i;let[1]='\0';
		strcat(str,"/");
		strcat(str,let);
		umask(0);
		mkdir(str,0777);
	}

	//druga seria katalogow .../UID32BASE/x/x
	for(i=65;i<91;i++) {
		strcpy(str,begin);
		let[0]=i;let[1]='\0';
		strcat(str,"/");
		strcat(str,let);
		strcpy(beg1,str);
		for(j=65;j<91;j++) {
		    strcpy(str,beg1);
		    let[0]=j;let[1]='\0';
		    strcat(str,"/");
		    strcat(str,let);
          umask(0);
		    mkdir(str,0777);
		}
	}

	//trzecia seria katalogow .../UID32BASE/x/x/x
	for(i=65;i<91;i++) {
		strcpy(str,begin);
		let[0]=i;let[1]='\0';
		strcat(str,"/");
		strcat(str,let);
		strcpy(beg1,str);
		for(j=65;j<91;j++) {
		    strcpy(str,beg1);
		    let[0]=j;let[1]='\0';
		    strcat(str,"/");
		    strcat(str,let);
		    strcpy(beg2,str);
			 for(k=65;k<91;k++){
			    strcpy(str,beg2);
		    	 let[0]=k;let[1]='\0';
		    	 strcat(str,"/");
		    	 strcat(str,let);
				 p = strchr(str, '\n');
				 if(p !=NULL) *p = '\0';
			    umask(0);
		    	 mkdir(str,0777);
				 strcpy(st, "rm -fr ");
			    strcat(st, str);
			    strcat(st, "/*");
//printf("ST: %s\n", st);
				 system(st);
			    //fp = popen(st, "r");
			    //if(fp != NULL) pclose(fp);
			}
		}
	}

}



//czysci baze PESEL
void clear_base_PESEL(void) {
	char str[126];
	strcpy(str, "rm -fr ");
	strcat(str, pesel_base);
	strcat(str, "/*");
//printf("STR: %s\n", str);
	FILE*fp = popen(str, "r");
	if(fp != NULL) pclose(fp);
}



//czysci baze START
void clear_base_START(void) {

	char str[200]="";
	char begin[200];
	char beg1[200];
	char beg2[200];
	char st[126];
	char let[3];
	int i,j,k;


	//budowanie dostepu do START
	strcat(str, start_base);		 
	strcpy(begin,str);
   umask(0);
	mkdir(str, 0777);
	strcat(str, "/name");
	remove(str); //usuwa poprzedni zapis jesli byl
	add_string_to_file(str, "START"); //zapisuje nazwe bazy




	//pierwsza seria katalogow .../START/x
	for(i=65;i<91;i++) {
		strcpy(str,begin);
		let[0]=i;let[1]='\0';
		strcat(str,"/");
		strcat(str,let);
	   umask(0);
		mkdir(str,0777);
	}

	//druga seria katalogow .../START/x/x
	for(i=65;i<91;i++) {
		strcpy(str,begin);
		let[0]=i;let[1]='\0';
		strcat(str,"/");
		strcat(str,let);
		strcpy(beg1,str);
		for(j=65;j<91;j++) {
		    strcpy(str,beg1);
		    let[0]=j;let[1]='\0';
		    strcat(str,"/");
		    strcat(str,let);
		    umask(0);
		    mkdir(str,0777);
		    }
	}

	//trzecia seria katalogow .../START/x/x/x
	for(i=65;i<91;i++) {
		strcpy(str,begin);
		let[0]=i;let[1]='\0';
		strcat(str,"/");
		strcat(str,let);
		strcpy(beg1,str);
		for(j=65;j<91;j++) {
		    strcpy(str,beg1);
		    let[0]=j;let[1]='\0';
		    strcat(str,"/");
		    strcat(str,let);
		    strcpy(beg2,str);
			for(k=65;k<91;k++){
			    strcpy(str,beg2);
		    	 let[0]=k;let[1]='\0';
		    	 strcat(str,"/");
		    	 strcat(str,let);
			    umask(0);
		    	 mkdir(str,0777);
				 //usuwa pliki SCODE, etc.
				 strcpy(st, "rm -fr ");
			    strcat(st, str);
			    strcat(st, "/*");
				 system(st);
			    //remove(SCODE)
			}
		}
	}
}
//------------------------------ END CLEAR BASE -------------------------------


//-------------------------------  LOG ADMIN ----------------------------------

int log_file(const char*pesel, const char*nick) {

	char st[80];
	char qstr[80];
	char str[80];
	char path[128];
	char *p;
	int i;
	int ch;
	long j;
   long fsize;
   long nlines;
	


	//buduje qstring (PESEL=..&NICK=..&)
	//zapis do pliku .../DISK/log.q
	strcpy(str, "PESEL=");
	strcat(str, pesel);
	strcat(str, "&NICK=");
	strcat(str, nick);
	strcat(str, "&");

	//dodaje czas logowania
	if (get_time(st) == 0) {
		strcat(str, st);
	}

	memset(qstr, '0', 80);
	memcpy(qstr, str, (size_t)strlen(str)+1); //string wsadzony razem z '\0'

	//tworzy sciezke dostepu do pliku do nadpisania
	strcpy(path, list_base);
	p = strchr(path, '/');
	if(p != NULL) *p = '\0';
	strcat(path, "/");
	strcat(path, LOGFILE);
	
 	ch = 0;
   FILE*fw = fopen(path, "r+");
		if(fw == NULL) goto label_1;
   	fsize = fseek(fw, 0L, SEEK_END);
    	fsize = ftell(fw);
    	fseek(fw, 0L, SEEK_SET);
    	nlines = fsize/60;

   for(i=0; i < (int)nlines; i++) {
		fread(str, 60, 1, fw); 
		select_value(str, "PESEL", st);
		if(strcmp(st, pesel) == 0)  {
			j = (long)i * 60;
			fseek(fw, j, SEEK_SET);
			fwrite(qstr, 60, 1, fw);
			fflush(fw);
			fclose(fw);
			return 1;		//znalazl, zmienil i wyskauje
		}	 
   }
	fclose(fw);    

label_1:

	//jak nie bylo znalezionego pesel to dopisuje do pliku
	fw = fopen(path, "a");
	if(fw == NULL) return -1;
	if(!ch) {
		fwrite(	qstr, 60, 1, fw);
		fflush(fw);
	}
	fclose(fw);

	return 1;
}



int read_log_file(const char*fname, const char*pesel,
					 const char*nick, char*str) {

	long fsize;
   long nlines;
	int i;
	char qstr[200];
	char b_pesel[15];
	char b_nick[80];
	char*p;

	strcpy(str, "null");

	FILE*fd = fopen(fname, "r");
	if(fd == NULL) return -1;

	fsize = fseek(fd, 0L, SEEK_END);
	fsize = ftell(fd);
	fseek(fd, 0L, SEEK_SET);
	nlines = fsize/60;

	for(i=0; i < (int)nlines; i++) {
		fread(qstr, 60, 1, fd);
		p = strrchr(qstr, '&');
		if(p != NULL) *(p+1) = '\0'; //konczy string

		if(strcmp(pesel, "null") != 0) {
			select_value(qstr, "PESEL", b_pesel);
			if(strcmp(b_pesel, pesel) == 0)  {
				strcpy(str, qstr);
				goto label_2;	
			}
		}
	
		if(strcmp(nick, "null") != 0) {
			select_value(qstr, "NICK", b_nick);
			if(strcmp(b_nick, nick) == 0)  {
				strcpy(str, qstr);
				goto label_2;	
			}
		}

	}

	
label_2: 

	fclose(fd);
	
	return 0;
}

//------------------------------- END LOG ADMIN ------------------------------- 
