/******************************************************************************
 *				ARCHI-CLIENT-SERVER
 *				    mserverUIDC
 *				   SERVER CENTRAL
 *				(multiple-server)
 *				     ver 0.5
 *
 * NOTE: aplikacja serwera wielozadaniowego dla komunikacji lokalnej
 * Kompilacja: make
 *
 ******************************************************************************/




#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/epoll.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>

#include <netinet/in.h> // dla detekcji adresu client
#include <arpa/inet.h>  //dla detkcji adresu client

#ifndef SELECT 
#define SELECT
#include "SRC/select_value.c" 
#endif

#ifndef READ_CONFIG 
#define READ_CONFIG
#include "SRC/read_config.c"
#endif


//--- HEADER SIZE IN THE BUFFER ---
#define HEADSIZE 512
//--- DATA SIZE IN THE BUFFER ---
#define DATASIZE 512
//--- SIZE OF THE BUFFER FOR DATA TRANSPORT ---
#define BUFSIZE HEADSIZE+DATASIZE
//--- NUMBER OF SOCKET (CLIENTS) FOR OBSERVATION ---
#define MAXEVENTS 64



#define PORT_TCPIP "8809" //definjujemy port nasluchiwania
char  port[20] = PORT_TCPIP;


//#define CONFIG_FILENAME "mserverUIDC.config"


//timeout nasluchu klienta
#define TIMEOUT_S 5 //rzuca polacznie po timeout (sekundy)
#define TIMEOUT_uS 0 //timeout mikrosekundy

#define QUIET 0 //0=off (tryb daemon), 1=on(tryb testowy)



#include "executorUIDC.c"






//-----------------------------------------------------------------------------
// -------------- function will handle connection for each client -------------
//table of tmp descritors
int dsc_table[MAXEVENTS];


//set descriptor to dsc_table
int set_ptr(int desc) {

   int i;

   for(i=0; i<MAXEVENTS; i++) {
     if(desc==dsc_table[i]) return i;
   }
   for(i=0; i < MAXEVENTS; i++) {
      if(dsc_table[i]==-1) {
         dsc_table[i]=desc;
         return i;
      }
   }
   return 0;
}


//clear descriptor in dsc_table
void clr_ptr(int desc) {
   int i;
   for(i=0; i < MAXEVENTS; i++) 
      if(desc==dsc_table[i]) {
         dsc_table[i]=-1;
         return;
      }
}



//set scket in non blocking mode
static int ake_socket_nonblocking(int sfd) {
   int flags, s;
   flags = fcntl(sfd, F_GETFL, 0);
   if (flags==-1) {
      perror("fcntl");
      return -1;
    }
   flags |= O_NONBLOCK;
   s = fcntl(sfd, F_SETFL, flags);
   if(s==-1) {
      perror("fcntl");
      return -1;
   }
   return 0;
}


//creates and binds socket
static int create_and_bind(char *port) {
   struct addrinfo hints;
   struct addrinfo*result, *rp;
   int s, sfd;

   memset(&hints, 0, sizeof (struct addrinfo));
   hints.ai_family = AF_UNSPEC;     // Return IPv4 and IPv6 choices 
   hints.ai_socktype = SOCK_STREAM; // TCP socket 
   hints.ai_flags = AI_PASSIVE;     // All interfaces 

   s = getaddrinfo(NULL, port, &hints, &result);
   if(s!=0) {
      if(test_mode)
         fprintf(stderr, "getaddrinfo: %s\n", gai_strerror (s));
      return -1;
    }
   for(rp=result; rp!=NULL; rp=rp->ai_next) {
      sfd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
      if (sfd == -1)
        continue;
      s = bind(sfd, rp->ai_addr, rp->ai_addrlen);
      if (s==0) {
          //managed to bind successfully
          break;
      }
      close (sfd);
   }
   if (rp==NULL) {
      if(test_mode)
         fprintf(stderr, "ERROR: can't bind\n");
      return -1;
   }
   freeaddrinfo (result);
   return sfd;
}

void error(const char*msg) {
   perror(msg);
   abort();
}




                         /* --- SERVER --- */

int server (void) {

   int done;
   int n, i, desc, j;
   int sfd, s;
   int efd;
   struct epoll_event event;
   struct epoll_event *events;

   struct sockaddr in_addr;
   socklen_t in_len;
   int infd;
   char hbuf[NI_MAXHOST], sbuf[NI_MAXSERV];


   // --- pobieranie adresu clienta ---
   char ip_client[INET_ADDRSTRLEN]; //<---------------------------------------------------- test
   struct sockaddr_in *caddress;
   struct in_addr ipaddr;
   // --- koniec testu poberania adresu clienta ---



   char*stupid_message = "I love you.";

   ssize_t count, tot[MAXEVENTS];
   char buf[BUFSIZE+100];
   char buffer[MAXEVENTS][BUFSIZE];

   //init data
   for(i=0; i<MAXEVENTS; i++) {tot[i]=0; dsc_table[i]=-1;}



   sfd = create_and_bind(port);
   if(sfd==-1) error("create_and_bind");

   s = ake_socket_nonblocking(sfd);
   if(s==-1) abort();

   s = listen(sfd, SOMAXCONN);
   if(s==-1) error("listen");

   efd = epoll_create1(0);
   if(efd==-1) error("epoll_create");
 
   event.data.fd = sfd;
   event.events = EPOLLIN | EPOLLET;
   s = epoll_ctl (efd, EPOLL_CTL_ADD, sfd, &event);
   if(s==-1) error("epoll_ctl");
  
   // Buffer where events are returned 
   events = (struct epoll_event*)calloc(MAXEVENTS, sizeof(event));

   // The event loop 
   while(1) {

      n = epoll_wait(efd, events, MAXEVENTS, -1);
      for(i = 0; i < n; i++) {
	   if((events[i].events & EPOLLERR) ||
              (events[i].events & EPOLLHUP) ||
              (!(events[i].events & EPOLLIN))) {
              //An error has occured on this fd, or the socket is not
              //ready for reading 
              if(test_mode)
	          fprintf(stderr, "epoll error\n");
	      close(events[i].data.fd);
	      continue;
           }
	  else if(sfd == events[i].data.fd) {
              // it is a notification on the listening socket, which
              // means one or more incoming connections. 
              while (1) {

                  in_len = sizeof(in_addr);
                  infd = accept(sfd, &in_addr, &in_len);


		 //< ------------------------------------------------- pobieranie adresu clienta
		 caddress = (struct sockaddr_in*)&in_addr; 
		 ipaddr = caddress->sin_addr;
		 inet_ntop( AF_INET, &ipaddr, ip_client, INET_ADDRSTRLEN );
		 //<------------------------------------------------- koniec pobierania adresu clienat



			
                  if(infd==-1) {
                      if((errno==EAGAIN) ||
                          (errno==EWOULDBLOCK)) {
                          //it is processed all incoming connections.
                          break;
                        }
                      else {
                          perror ("accept");
                          break;
                        }
                  }

                  s = getnameinfo(&in_addr, in_len,
                                  hbuf, sizeof(hbuf),
                                  sbuf, sizeof(sbuf),
                                  NI_NUMERICHOST | NI_NUMERICSERV);
                  if(s==0) {
		      if(test_mode)
                          printf("Accepted connection on descriptor %d "
                             "(host=%s, port=%s)\n", infd, hbuf, sbuf);
                  }

                  //make the incoming socket non-blocking and add it to the
                  //list of fds to monitor 
                  s = ake_socket_nonblocking(infd);
                  if(s==-1) abort();

                  event.data.fd = infd;
                  event.events = EPOLLIN | EPOLLET;
                  s = epoll_ctl(efd, EPOLL_CTL_ADD, infd, &event);
                  if(s==-1) error("epoll_ctl");
              }
              continue;
            }
          else {

              //There is data on the fd waiting to be read. Read and
              //display it. It must must read whatever data is available
              //completely, as we are running in edge-triggered mode
              //and won't get a notification again for the same
              //data. 

              done = 0;



    

              while (1) {

                  //count = read(events[i].data.fd, buf, sizeof(buf));
                  desc=events[i].data.fd;
		  j=set_ptr(desc);
		  count = recv(desc, buf, sizeof(buf), 0);
		  if(count>0) {
			//add missing part of message
			if(tot[j]<=BUFSIZE)
			    memcpy(&buffer[j][0]+tot[j], buf, (size_t)count);
                        tot[j]+=count;
			//Reset connection if message is too long
                        if(tot[j]>BUFSIZE) {
                           s= write(desc, stupid_message, 
                               strlen(stupid_message)+1);
                           if(s==-1) error("write");
			   done = 1;
                           tot[j]= 0;
		           clr_ptr(desc);
                           break;
                        }
                  }                  


                  if(count==-1) {
                      //if errno == EAGAIN, that means we have read all
                      //data, so go back to the main loop. 
                      if(errno!=EAGAIN) {
                          perror ("read");
                          done = 1;
		          tot[j]=0;
			  clr_ptr(desc);
                      }
                      break;
                  }
                  else if(count==0) {
                      //EOF, the remote has closed the
                      //connection.
                      done = 1;
                      tot[j]= 0;
		      clr_ptr(desc);
                      break;
                    }

		  
		  //message has correct lenght (executor)
		  //analizator(buf,buf); 
		  if(tot[j]==BUFSIZE) {
		        	
 		      //<----------------------------------- input to executor
		      executor(buffer[j], ip_client);
		      //-----------------------------------> output from executor

                      //write the buffer to output
		      s = write(desc, 
                                buffer[j], sizeof(buffer[j]));
                      if(s==-1) error("write");
		      tot[j]=0;
		  }

                  //Close connection if message is too short
                  if(tot[j]<BUFSIZE && tot[j]>0) {
                      s= write(desc, stupid_message, 
                               strlen(stupid_message)+1);
                      if(s==-1) error("write");
		      tot[j]=0;
		      done = 1;
                      break;
                  }

              }


              if (done) {
		  if(test_mode)
                       printf ("Closed connection on descriptor %d\n",
                          events[i].data.fd);

                  //closing the descriptor will make epoll remove it
                  //from the set of descriptors which are monitored. 
                  close (events[i].data.fd);
                  tot[j]=0;
		  clr_ptr(desc);
              }
           }
        }
    }

   free (events);

   close (sfd);

   return EXIT_SUCCESS;
}
//------------------------- END handle connection procedure -------------------
//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
//--------------------------- Main Loop check mserverUID ----------------------

int main(int argc, char * argv[]) {

   //read IP i PORT z pliku konfigurayjnego
   read_config();

   //port pobrany z mserverUIDC.config
   if(port_central == 0) {
		printf("\n\n   Error: Incorrect port number in config file\n\n");
		return 0;
	}
   sprintf(port, "%d", port_central);
	 

   printf("\n ... mserver_UIDC works ...\n\n");

   server();

   return 0;
}
//--------------------------------------- END ---------------------------------
